;; magma-mode.el --- MAGMA code editing commands for Emacs
;;
;; Maintainer: David R. Kohel (kohel@maths.usyd.edu.au)
;; Modified from magma-mode script of William A. Stein with 
;; contributions from Pierrick Gaudry.
;;
;; Defines a magma-mode under emacs.
;; Magma commands, builtins, verbose modes, and types are hilighted 
;; according to external database files.  For efficiency these may be 
;; much smaller than the complete enumerations of these types. 
;; The lists below, from which magma-mode.el is generated, should be 
;; modified for personal use, 
;;
;; intrinsics.in
;; categories.in
;; reserved.in
;; verbose.in
;; operators.in
;; punctuation.in
;;
;; Generating scripts make_magma_mode and build_faces by David R. Kohel. 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;; Indentation block
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;   

;;; hacked by P. Gaudry from:
;;;;;;;;;; sh-mode.el
;;;;;;;;;; LCD Archive Entry:
;;;;;;;;;; sh-mode|Thomas W. Strong, Jr.|strong+@cmu.edu|
;;;;;;;;;; Beginning of an sh mode.|
;;;;;;;;;; 92-11-15||~/modes/sh-mode.el.Z|
;;;;;;;;;; hacked out of forth.el from the tile-forth package
;;;;;;;;;; comments/complaints/flames to strong+@cmu.edu

(defvar magma-mode-positives
  " intrinsic function procedure for repeat while then else case when "
  "Words which will cause the indent-level to be incremented on the next line.
OBSERVE: Words in magma-mode-positives must be surrounded by spaces.")

(defvar magma-mode-negatives 
  " else elif when until end "
  "Words which will cause the indent-level to be decremented on the current line.
OBSERVE: Words in magma-mode-negatives must be surrounded by spaces.")

(defvar magma-mode-zeroes 
  " intrinsic "
  "Words which causes the indent to go to zero")

(defvar magma-mode-abbrev-table nil
  "Abbrev table in use in magma-mode buffers.")
(define-abbrev-table 'magma-mode-abbrev-table ())

(defvar magma-mode-map nil
  "Keymap used in magma mode.")

(if (not magma-mode-map)
    (setq magma-mode-map (make-sparse-keymap)))

(define-key magma-mode-map "\t" 'magma-indent-command)
(define-key magma-mode-map "\C-m" 'reindent-then-newline-and-indent)

(defvar magma-indent-level 4
  "*Indentation of magma statements.")

(defun magma-mode-variables ()
  (setq local-abbrev-table magma-mode-abbrev-table)
  (make-local-variable 'indent-line-function)
  (setq indent-line-function 'magma-indent-line)
  (make-local-variable 'require-final-newline)
  (setq require-final-newline t)
  (make-local-variable 'parse-sexp-ignore-comments)
  (setq parse-sexp-ignore-comments t))
  
(defun magma-current-indentation ()
  (save-excursion
    (beginning-of-line)
    (back-to-indentation)
    (current-column)))

(defun magma-delete-indentation ()
  (let ((b nil) (m nil))
    (save-excursion
      (beginning-of-line)
      (setq b (point))
      (back-to-indentation)
      (setq m (point)))
    (delete-region b m)))

(defun magma-indent-line (&optional flag)
  "Correct indentation of the current magma line."
  (if (not (magma-line-is-starred-comment))  
    (let ((x (magma-calculate-indent))) (magma-indent-to x))))
  
(defun magma-indent-command ()
  (interactive) (magma-indent-line t))

(defun magma-indent-to (x)
  (let ((p nil))
    (setq p (- (current-column) (magma-current-indentation)))
    (magma-delete-indentation)
    (beginning-of-line)
    (indent-to x)
    (if (> p 0) (forward-char p))))

(defun magma-calculate-indent ()
  (let ((w1 nil) (indent 0) (centre 0) (e (buffer-size)))
    (save-excursion
      (beginning-of-line)
      (if (< (point) 2) 
        ()
      ;else
        (skip-chars-backward " \t\n" 0)
        (beginning-of-line)
        (back-to-indentation)
        (setq indent (current-column))
        (setq centre indent)
        (setq indent (+ indent (magma-sum-line-indentation)))))
    (save-excursion
      (beginning-of-line)
      (if (< (point) 2) 
        ()
      ;else
        (back-to-indentation)
        (let ((p (point)))
          (skip-chars-forward "^ \t\n" e)
          (setq w1 (buffer-substring p (point))))))
    (if (> (- indent centre) magma-indent-level)
      (setq indent (+ centre magma-indent-level)))
    (if (> (- centre indent) magma-indent-level)
      (setq indent (- centre magma-indent-level)))
    (if (< indent 0) (setq indent 0))
    (if (magma-line-is-signature)
      ()
    ;else
      (setq indent (- indent
        (if (string-match 
          (regexp-quote (concat " " w1 " ")) magma-mode-negatives)
          (if (string= w1 "else") 
            (if (magma-else-is-if-then) magma-indent-level 0)
          ;else
            magma-indent-level) 
         ;else
         0)))
      (if (string-match (regexp-quote 
        (concat " " w1 " ")) magma-mode-zeroes)
        (setq indent 0)))
    indent))

(defun magma-else-is-if-then ()
  (let ((b0 (- (point) 1)) (b1 0) (b2 0) (b3 0))
    ; This needs to be more sophisticated. In particular it needs 
    ; to look for the previous "else", "then", or "select", and 
    ; pair each "else" with its preceeding partner.
    (goto-char b0)
    (word-search-backward "select" 0 t)
    (setq b1 (point)) (if (= b1 b0) (setq b1 0))
    ;
    (goto-char b0)
    (word-search-backward "then" 0 t) 
    (setq b2 (point)) (if (= b2 b0) (setq b2 0))
    ;
    (if (< b1 b2)
      ()
      (goto-char b0)
      (word-search-backward "else" 0 t) 
      (setq b3 (point)) (if (= b3 b0) (setq b3 0))
      (setq b2 (max b2 b3)))
    ;
    (goto-char (+ b0 1))
    (< b1 b2)))

(defun magma-line-is-signature ()
  "True if and only if the line is in a signature."
  (let ((b0 (point)) (e0 (buffer-size)))
    (if (search-backward "intrinsic " 0 t)
      (let ((b1 (point)) (b2 0))
        (skip-chars-forward "^{" e0)
        (let ((n 1))
          (while (and (> n 0) (< (point) e0))
	    (forward-char 1)
            (skip-chars-forward "^{}" e0)
            (if (looking-at "}") 
              (setq n (+ n -1))
            ;else
              (setq n (+ n 1)))))
        (setq b2 (point))
        (goto-char b0)
        (and (< b1 b0) (< b0 b2)))
    ;else
      nil)))

(defun magma-line-is-terminated ()
  "True if and only if the line is terminated."
  ; i.e. ends in a semicolon or is a complete statement. 
  (let ((e nil) (w nil) (terminated t)) 
    (if (or (magma-line-is-signature) (magma-line-is-starred-comment)) 
      t
    ;else
      (end-of-line) 
      (setq e (point))
      (beginning-of-line) 
      (while (< (point) e)
        (setq w (magma-next-word))
	(if (string-match "*.\*/" w) (setq w ";"))
        ;(princ (concat w "|"))
        (if (not (or (or (or (string-match 
          (regexp-quote (concat " " w " ")) magma-mode-positives))
          (string= "->" w)) (string-match ".?;" w)))
          (if (or (or (or (= (length w) 0) (string= "\"\"" w))
            (string= "{}" w)) (string= "//" w))
            (skip-chars-forward " \t\n" e)
          ;else
	    (setq terminated ())
	    (skip-chars-forward " \t\n" e))
        ;else
          (if (string= "else" w) 
            (if (not (magma-else-is-if-then)) 
              (let ()
                (setq terminated ())
                (skip-chars-forward " \t\n" e))
            ;else
              (end-of-line)       
              (setq terminated t))
          ;else
            (end-of-line)       
            (setq terminated t))))
      ;(princ "(") (princ terminated) (princ ")\n")
      terminated)))

(defun magma-line-is-starred-comment ()
  "True if line is embedded between /* and */."
  (let ((b (line-beginning-position)) (e (line-end-position)) (b1 0) (b2 0))
    (save-excursion 
      (beginning-of-line)
      (search-backward "/\*" (point-min) t)
      (setq b1 (point))
      (search-forward "\*/" (point-max) t)
      (setq b2 (+ (point) 1)))
    ;(princ b1) (princ " < ") (princ b) (princ " <= ") 
    ;(princ e) (princ " < ") (princ b2) (princ "?\n")
    (and (< b1 b) (< e b2))))

(defun magma-sum-line-indentation ()
  "Add up the positive and negative weights of all words on the current line."
  ; Analyzes previous two lines to determine indentation 
  (let ((b (point)) (e nil) (sum 0) (w nil) (t1 nil) (t2 nil) (first t))
    (if (< b 2)
      ()
    ;else 
      (end-of-line) 
      (setq e (point))
      ;
      (goto-char b)
      (setq t1 (magma-line-is-terminated))
      ;
      (goto-char b)
      (beginning-of-line) 
      (skip-chars-backward " \t\n" 0) 
      (beginning-of-line) 
      (setq t2 (or (< (point) 2) (magma-line-is-terminated)))
      (if (and (not t1) t2) 
        (setq sum magma-indent-level))
      (if (and t1 (not t2)) 
        (setq sum (- 0 magma-indent-level)))
      (goto-char b)
      (if (not (magma-line-is-signature)) 
        (while (< (point) e)
          (setq w (magma-next-word))
          (setq t1 (string-match 
            (regexp-quote (concat " " w " ")) magma-mode-positives))
          (if (and t1 (string= "else" w)) 
            (if (not (magma-else-is-if-then)) (setq t1 nil)))
          (setq t2 (string-match 
            (regexp-quote (concat " " w " ")) magma-mode-negatives))
          (if (and t2 (string= "else" w)) 
            (if (not (magma-else-is-if-then)) (setq t2 nil)))
          (if (and t1 t2) (setq sum (+ sum magma-indent-level)))
          (if t1 (setq sum (+ sum magma-indent-level)))
          (if (and t2 (not first))
            (setq sum (- sum magma-indent-level)))
          (skip-chars-forward " \t\n" e)
          (setq first nil)))
      (goto-char b))
    sum))

(defun magma-next-word ()
  "Return the next magma-word. Skip anything enclosed in double quotes."
  (let ((e (buffer-size)) (w nil))
    (while (not w)
      (skip-chars-forward " \t\n" e)
      (if (string= "\/\*" (buffer-substring (point) (+ 2 (point))))
        (let ()
	  (search-forward "\*/" e)
	  (setq w "/\*\*/"))
      ;else
      (if (not (looking-at "\""))
        (if (not (looking-at "{"))
          (if (not (looking-at "//"))
            (let ((p (point)))
              (skip-chars-forward "^ \"\t\n" e)
              (setq w (buffer-substring p (point)))
              (if (not (or (string= w "select") (string= w "intrinsic")))
                ()
              ;else
                (if (string= w "select")
                  (skip-chars-forward "^;" e)
                ;else "intrinsic"
                  (skip-chars-forward "^{" e))))
          ;else (looking-at "//")
            (skip-chars-forward "^\n" e)
            (setq w "//"))
        ;else (looking-at "{")
          (let ((n 1))
            (while (and (< 0 n) (< (point) e))
              (forward-char 1)
              (skip-chars-forward "^{}" e)
              (if (looking-at "}")
                (setq n (- n 1))
              ;else
                (setq n (+ n 1)))))
          (setq w "{}"))
      ;else (looking-at "\"")
        (forward-char 1)
        (skip-chars-forward "^\"" e)
        (forward-char 1)
        (setq w "\"\""))))
    w))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; End indentation block
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defvar magma-mode-abbrev-table nil
  "Abbrev table in use in Magma-mode buffers.")
(define-abbrev-table 'magma-mode-abbrev-table ())

(defvar magma-mode-syntax-table nil
  "Syntax table in use in Magma-mode buffers.")

(if magma-mode-syntax-table
    ()
  (setq magma-mode-syntax-table (make-syntax-table))
  (modify-syntax-entry ?\\ "\\"  magma-mode-syntax-table)
  (modify-syntax-entry ?/ ". 14" magma-mode-syntax-table)
  (modify-syntax-entry ?* ". 23" magma-mode-syntax-table)
  (modify-syntax-entry ?+ "."    magma-mode-syntax-table)
  (modify-syntax-entry ?- "."    magma-mode-syntax-table)
  (modify-syntax-entry ?= "."    magma-mode-syntax-table)
  (modify-syntax-entry ?% "."    magma-mode-syntax-table)
  (modify-syntax-entry ?< "."    magma-mode-syntax-table)
  (modify-syntax-entry ?> "."    magma-mode-syntax-table)
  (modify-syntax-entry ?& "."    magma-mode-syntax-table)
  (modify-syntax-entry ?| "."    magma-mode-syntax-table)
  (modify-syntax-entry ?\' "."   magma-mode-syntax-table))

(defun magma-mode ()
  "Major mode for editing MAGMA code.
This is much like C mode except for the syntax of comments.  It uses
the same keymap as C mode and has the same variables for customizing
indentation.  It has its own abbrev table and its own syntax table.

Turning on MAGMA mode calls the value of the variable `magma-mode-hook'
with no args, if that value is non-nil."
  (interactive)
  (make-local-variable 'font-lock-defaults)
  (setq font-lock-defaults '(magma-font-lock-keywords nil nil ((?_ . "w"))))
  (use-local-map magma-mode-map)
  (setq mode-name "Magma")
  (setq major-mode 'magma-mode)
  (magma-mode-variables) ;; gaudry
  (set-syntax-table magma-mode-syntax-table)
  (make-local-variable 'paragraph-start)
  (setq paragraph-start (concat "$\\|" page-delimiter))
  (make-local-variable 'paragraph-separate)
  (setq paragraph-separate paragraph-start)
  (make-local-variable 'paragraph-ignore-fill-prefix)
  (setq paragraph-ignore-fill-prefix t)
  (make-local-variable 'fill-paragraph-function)
  (setq fill-paragraph-function 'c-fill-paragraph)
  (make-local-variable 'require-final-newline)
  (setq require-final-newline t)
  (make-local-variable 'outline-regexp)
  (setq outline-regexp "[^#\n\^M]")
  (make-local-variable 'comment-start)
  (setq comment-start "/*")
  (make-local-variable 'comment-end)
  (setq comment-end "*/")
  (make-local-variable 'comment-column)
  (setq comment-column 32)
  (make-local-variable 'comment-start-skip)
  (setq comment-start-skip "/\\*+ *")
  (make-local-variable 'comment-indent-function)
  (setq comment-indent-function 'c-comment-indent)
  (make-local-variable 'comment-multi-line)
  (setq comment-multi-line t)
  (make-local-variable 'parse-sexp-ignore-comments)
  (setq parse-sexp-ignore-comments t)
  (make-local-variable 'imenu-generic-expression)

  (run-hooks 'magma-mode-hook))    

(provide 'magma-mode)
