;; magma-mode.el --- MAGMA code editing commands for Emacs
;;
;; Maintainer: David R. Kohel (kohel@maths.usyd.edu.au)
;; Modified from magma-mode script of William A. Stein with 
;; contributions from Pierrick Gaudry.
;;
;; Defines a magma-mode under emacs.
;; Magma commands, builtins, verbose modes, and types are hilighted 
;; according to external database files.  For efficiency these may be 
;; much smaller than the complete enumerations of these types. 
;; The lists below, from which magma-mode.el is generated, should be 
;; modified for personal use, 
;;
;; intrinsics.in
;; categories.in
;; reserved.in
;; verbose.in
;; operators.in
;; punctuation.in
;;
;; Generating scripts make_magma_mode and build_faces by David R. Kohel. 

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; 
;; Indentation block
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;   

;;; hacked by P. Gaudry from:
;;;;;;;;;; sh-mode.el
;;;;;;;;;; LCD Archive Entry:
;;;;;;;;;; sh-mode|Thomas W. Strong, Jr.|strong+@cmu.edu|
;;;;;;;;;; Beginning of an sh mode.|
;;;;;;;;;; 92-11-15||~/modes/sh-mode.el.Z|
;;;;;;;;;; hacked out of forth.el from the tile-forth package
;;;;;;;;;; comments/complaints/flames to strong+@cmu.edu

(defvar magma-mode-positives
  " intrinsic function procedure for repeat while then else case when "
  "Words which will cause the indent-level to be incremented on the next line.
OBSERVE: Words in magma-mode-positives must be surrounded by spaces.")

(defvar magma-mode-negatives 
  " else elif when until end "
  "Words which will cause the indent-level to be decremented on the current line.
OBSERVE: Words in magma-mode-negatives must be surrounded by spaces.")

(defvar magma-mode-zeroes 
  " intrinsic "
  "Words which causes the indent to go to zero")

(defvar magma-mode-abbrev-table nil
  "Abbrev table in use in magma-mode buffers.")
(define-abbrev-table 'magma-mode-abbrev-table ())

(defvar magma-mode-map nil
  "Keymap used in magma mode.")

(if (not magma-mode-map)
    (setq magma-mode-map (make-sparse-keymap)))

(define-key magma-mode-map "\t" 'magma-indent-command)
(define-key magma-mode-map "\C-m" 'reindent-then-newline-and-indent)

(defvar magma-indent-level 4
  "*Indentation of magma statements.")

(defun magma-mode-variables ()
  (setq local-abbrev-table magma-mode-abbrev-table)
  (make-local-variable 'indent-line-function)
  (setq indent-line-function 'magma-indent-line)
  (make-local-variable 'require-final-newline)
  (setq require-final-newline t)
  (make-local-variable 'parse-sexp-ignore-comments)
  (setq parse-sexp-ignore-comments t))
  
(defun magma-current-indentation ()
  (save-excursion
    (beginning-of-line)
    (back-to-indentation)
    (current-column)))

(defun magma-delete-indentation ()
  (let ((b nil) (m nil))
    (save-excursion
      (beginning-of-line)
      (setq b (point))
      (back-to-indentation)
      (setq m (point)))
    (delete-region b m)))

(defun magma-indent-line (&optional flag)
  "Correct indentation of the current magma line."
  (if (not (magma-line-is-starred-comment))  
    (let ((x (magma-calculate-indent))) (magma-indent-to x))))
  
(defun magma-indent-command ()
  (interactive) (magma-indent-line t))

(defun magma-indent-to (x)
  (let ((p nil))
    (setq p (- (current-column) (magma-current-indentation)))
    (magma-delete-indentation)
    (beginning-of-line)
    (indent-to x)
    (if (> p 0) (forward-char p))))

(defun magma-calculate-indent ()
  (let ((w1 nil) (indent 0) (centre 0) (e (buffer-size)))
    (save-excursion
      (beginning-of-line)
      (if (< (point) 2) 
        ()
      ;else
        (skip-chars-backward " \t\n" 0)
        (beginning-of-line)
        (back-to-indentation)
        (setq indent (current-column))
        (setq centre indent)
        (setq indent (+ indent (magma-sum-line-indentation)))))
    (save-excursion
      (beginning-of-line)
      (if (< (point) 2) 
        ()
      ;else
        (back-to-indentation)
        (let ((p (point)))
          (skip-chars-forward "^ \t\n" e)
          (setq w1 (buffer-substring p (point))))))
    (if (> (- indent centre) magma-indent-level)
      (setq indent (+ centre magma-indent-level)))
    (if (> (- centre indent) magma-indent-level)
      (setq indent (- centre magma-indent-level)))
    (if (< indent 0) (setq indent 0))
    (if (magma-line-is-signature)
      ()
    ;else
      (setq indent (- indent
        (if (string-match 
          (regexp-quote (concat " " w1 " ")) magma-mode-negatives)
          (if (string= w1 "else") 
            (if (magma-else-is-if-then) magma-indent-level 0)
          ;else
            magma-indent-level) 
         ;else
         0)))
      (if (string-match (regexp-quote 
        (concat " " w1 " ")) magma-mode-zeroes)
        (setq indent 0)))
    indent))

(defun magma-else-is-if-then ()
  (let ((b0 (- (point) 1)) (b1 0) (b2 0) (b3 0))
    ; This needs to be more sophisticated. In particular it needs 
    ; to look for the previous "else", "then", or "select", and 
    ; pair each "else" with its preceeding partner.
    (goto-char b0)
    (word-search-backward "select" 0 t)
    (setq b1 (point)) (if (= b1 b0) (setq b1 0))
    ;
    (goto-char b0)
    (word-search-backward "then" 0 t) 
    (setq b2 (point)) (if (= b2 b0) (setq b2 0))
    ;
    (if (< b1 b2)
      ()
      (goto-char b0)
      (word-search-backward "else" 0 t) 
      (setq b3 (point)) (if (= b3 b0) (setq b3 0))
      (setq b2 (max b2 b3)))
    ;
    (goto-char (+ b0 1))
    (< b1 b2)))

(defun magma-line-is-signature ()
  "True if and only if the line is in a signature."
  (let ((b0 (point)) (e0 (buffer-size)))
    (if (search-backward "intrinsic " 0 t)
      (let ((b1 (point)) (b2 0))
        (skip-chars-forward "^{" e0)
        (let ((n 1))
          (while (and (> n 0) (< (point) e0))
	    (forward-char 1)
            (skip-chars-forward "^{}" e0)
            (if (looking-at "}") 
              (setq n (+ n -1))
            ;else
              (setq n (+ n 1)))))
        (setq b2 (point))
        (goto-char b0)
        (and (< b1 b0) (< b0 b2)))
    ;else
      nil)))

(defun magma-line-is-terminated ()
  "True if and only if the line is terminated."
  ; i.e. ends in a semicolon or is a complete statement. 
  (let ((e nil) (w nil) (terminated t)) 
    (if (or (magma-line-is-signature) (magma-line-is-starred-comment)) 
      t
    ;else
      (end-of-line) 
      (setq e (point))
      (beginning-of-line) 
      (while (< (point) e)
        (setq w (magma-next-word))
	(if (string-match "*.\*/" w) (setq w ";"))
        ;(princ (concat w "|"))
        (if (not (or (or (or (string-match 
          (regexp-quote (concat " " w " ")) magma-mode-positives))
          (string= "->" w)) (string-match ".?;" w)))
          (if (or (or (or (= (length w) 0) (string= "\"\"" w))
            (string= "{}" w)) (string= "//" w))
            (skip-chars-forward " \t\n" e)
          ;else
	    (setq terminated ())
	    (skip-chars-forward " \t\n" e))
        ;else
          (if (string= "else" w) 
            (if (not (magma-else-is-if-then)) 
              (let ()
                (setq terminated ())
                (skip-chars-forward " \t\n" e))
            ;else
              (end-of-line)       
              (setq terminated t))
          ;else
            (end-of-line)       
            (setq terminated t))))
      ;(princ "(") (princ terminated) (princ ")\n")
      terminated)))

(defun magma-line-is-starred-comment ()
  "True if line is embedded between /* and */."
  (let ((b (line-beginning-position)) (e (line-end-position)) (b1 0) (b2 0))
    (save-excursion 
      (beginning-of-line)
      (search-backward "/\*" (point-min) t)
      (setq b1 (point))
      (search-forward "\*/" (point-max) t)
      (setq b2 (+ (point) 1)))
    ;(princ b1) (princ " < ") (princ b) (princ " <= ") 
    ;(princ e) (princ " < ") (princ b2) (princ "?\n")
    (and (< b1 b) (< e b2))))

(defun magma-sum-line-indentation ()
  "Add up the positive and negative weights of all words on the current line."
  ; Analyzes previous two lines to determine indentation 
  (let ((b (point)) (e nil) (sum 0) (w nil) (t1 nil) (t2 nil) (first t))
    (if (< b 2)
      ()
    ;else 
      (end-of-line) 
      (setq e (point))
      ;
      (goto-char b)
      (setq t1 (magma-line-is-terminated))
      ;
      (goto-char b)
      (beginning-of-line) 
      (skip-chars-backward " \t\n" 0) 
      (beginning-of-line) 
      (setq t2 (or (< (point) 2) (magma-line-is-terminated)))
      (if (and (not t1) t2) 
        (setq sum magma-indent-level))
      (if (and t1 (not t2)) 
        (setq sum (- 0 magma-indent-level)))
      (goto-char b)
      (if (not (magma-line-is-signature)) 
        (while (< (point) e)
          (setq w (magma-next-word))
          (setq t1 (string-match 
            (regexp-quote (concat " " w " ")) magma-mode-positives))
          (if (and t1 (string= "else" w)) 
            (if (not (magma-else-is-if-then)) (setq t1 nil)))
          (setq t2 (string-match 
            (regexp-quote (concat " " w " ")) magma-mode-negatives))
          (if (and t2 (string= "else" w)) 
            (if (not (magma-else-is-if-then)) (setq t2 nil)))
          (if (and t1 t2) (setq sum (+ sum magma-indent-level)))
          (if t1 (setq sum (+ sum magma-indent-level)))
          (if (and t2 (not first))
            (setq sum (- sum magma-indent-level)))
          (skip-chars-forward " \t\n" e)
          (setq first nil)))
      (goto-char b))
    sum))

(defun magma-next-word ()
  "Return the next magma-word. Skip anything enclosed in double quotes."
  (let ((e (buffer-size)) (w nil))
    (while (not w)
      (skip-chars-forward " \t\n" e)
      (if (string= "\/\*" (buffer-substring (point) (+ 2 (point))))
        (let ()
	  (search-forward "\*/" e)
	  (setq w "/\*\*/"))
      ;else
      (if (not (looking-at "\""))
        (if (not (looking-at "{"))
          (if (not (looking-at "//"))
            (let ((p (point)))
              (skip-chars-forward "^ \"\t\n" e)
              (setq w (buffer-substring p (point)))
              (if (not (or (string= w "select") (string= w "intrinsic")))
                ()
              ;else
                (if (string= w "select")
                  (skip-chars-forward "^;" e)
                ;else "intrinsic"
                  (skip-chars-forward "^{" e))))
          ;else (looking-at "//")
            (skip-chars-forward "^\n" e)
            (setq w "//"))
        ;else (looking-at "{")
          (let ((n 1))
            (while (and (< 0 n) (< (point) e))
              (forward-char 1)
              (skip-chars-forward "^{}" e)
              (if (looking-at "}")
                (setq n (- n 1))
              ;else
                (setq n (+ n 1)))))
          (setq w "{}"))
      ;else (looking-at "\"")
        (forward-char 1)
        (skip-chars-forward "^\"" e)
        (forward-char 1)
        (setq w "\"\""))))
    w))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; End indentation block
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defvar magma-mode-abbrev-table nil
  "Abbrev table in use in Magma-mode buffers.")
(define-abbrev-table 'magma-mode-abbrev-table ())

(defvar magma-mode-syntax-table nil
  "Syntax table in use in Magma-mode buffers.")

(if magma-mode-syntax-table
    ()
  (setq magma-mode-syntax-table (make-syntax-table))
  (modify-syntax-entry ?\\ "\\"  magma-mode-syntax-table)
  (modify-syntax-entry ?/ ". 14" magma-mode-syntax-table)
  (modify-syntax-entry ?* ". 23" magma-mode-syntax-table)
  (modify-syntax-entry ?+ "."    magma-mode-syntax-table)
  (modify-syntax-entry ?- "."    magma-mode-syntax-table)
  (modify-syntax-entry ?= "."    magma-mode-syntax-table)
  (modify-syntax-entry ?% "."    magma-mode-syntax-table)
  (modify-syntax-entry ?< "."    magma-mode-syntax-table)
  (modify-syntax-entry ?> "."    magma-mode-syntax-table)
  (modify-syntax-entry ?& "."    magma-mode-syntax-table)
  (modify-syntax-entry ?| "."    magma-mode-syntax-table)
  (modify-syntax-entry ?\' "."   magma-mode-syntax-table))

(defun magma-mode ()
  "Major mode for editing MAGMA code.
This is much like C mode except for the syntax of comments.  It uses
the same keymap as C mode and has the same variables for customizing
indentation.  It has its own abbrev table and its own syntax table.

Turning on MAGMA mode calls the value of the variable `magma-mode-hook'
with no args, if that value is non-nil."
  (interactive)
  (make-local-variable 'font-lock-defaults)
  (setq font-lock-defaults '(magma-font-lock-keywords nil nil ((?_ . "w"))))
  (use-local-map magma-mode-map)
  (setq mode-name "Magma")
  (setq major-mode 'magma-mode)
  (magma-mode-variables) ;; gaudry
  (set-syntax-table magma-mode-syntax-table)
  (make-local-variable 'paragraph-start)
  (setq paragraph-start (concat "$\\|" page-delimiter))
  (make-local-variable 'paragraph-separate)
  (setq paragraph-separate paragraph-start)
  (make-local-variable 'paragraph-ignore-fill-prefix)
  (setq paragraph-ignore-fill-prefix t)
  (make-local-variable 'fill-paragraph-function)
  (setq fill-paragraph-function 'c-fill-paragraph)
  (make-local-variable 'require-final-newline)
  (setq require-final-newline t)
  (make-local-variable 'outline-regexp)
  (setq outline-regexp "[^#\n\^M]")
  (make-local-variable 'comment-start)
  (setq comment-start "/*")
  (make-local-variable 'comment-end)
  (setq comment-end "*/")
  (make-local-variable 'comment-column)
  (setq comment-column 32)
  (make-local-variable 'comment-start-skip)
  (setq comment-start-skip "/\\*+ *")
  (make-local-variable 'comment-indent-function)
  (setq comment-indent-function 'c-comment-indent)
  (make-local-variable 'comment-multi-line)
  (setq comment-multi-line t)
  (make-local-variable 'parse-sexp-ignore-comments)
  (setq parse-sexp-ignore-comments t)
  (make-local-variable 'imenu-generic-expression)

  (run-hooks 'magma-mode-hook))    

(provide 'magma-mode)
;; Regexps
(defconst magma-font-lock-keywords
  (eval-when-compile
    (list
      ;; Highlight name when user is defining a function, procedure or intrinsic
      '("^[ \t]*\\(function\\|intrinsic\\|procedure\\)\\>[ \t]*\\(\\sw+\\)?"
        (1 font-lock-keyword-face) (2 font-lock-function-name-face nil t))

      ;; Comments below intrinsics works a little.
      (cons (concat "{\\([^}]\\|\n\\)*\}") 'font-lock-comment-face)
      ;; C++-style comments.
      (cons (concat "//.*") 'font-lock-comment-face)

      ;; Intrinsics.
      (cons (concat "\\<\\("
        "AGM\\|AbelianBasis\\|"
        "AbelianExtension\\|AbelianGroup\\|"
        "AbelianInvariants\\|AbelianMonoid\\|"
        "AbelianQuotient\\|AbelianSection\\|"
        "AbelianSubgroups\\|AbelianpExtension\\|"
        "Abs\\|AbsoluteDegree\\|"
        "AbsoluteField\\|AbsoluteInvariants\\|"
        "AbsoluteNorm\\|AbsoluteOrder\\|"
        "AbsolutePolynomial\\|AbsolutePrecision\\|"
        "AbsoluteTrace\\|AbsoluteValue\\|"
        "AbsoluteValues\\|AcceptingStates\\|"
        "Accepts\\|ActingWord\\|"
        "Action\\|ActionGenerator\\|"
        "ActionGenerators\\|ActionImage\\|"
        "ActionKernel\\|ActionModule\\|"
        "AdditiveGroup\\|AdjacencyMatrix\\|"
        "Adjoint\\|AdjointGraph\\|"
        "Adjoints\\|Advance\\|"
        "AffineAction\\|AffineAlgebra\\|"
        "AffineAmbient\\|AffineCurve\\|"
        "AffineImage\\|AffineKernel\\|"
        "AffinePatch\\|AffinePlane\\|"
        "AffineSpace\\|Agemo\\|"
        "Algebra\\|AlgebraMap\\|"
        "AlgebraicClosure\\|AlgebraicFunction\\|"
        "AlgebraicRelations\\|AllowableSubgroup\\|"
        "AlmostSimpleGroupDatabase\\|Alphabet\\|"
        "Alt\\|AlternantCode\\|"
        "AlternatingGroup\\|AlternatingModule\\|"
        "AlternatingSum\\|AltinokNumber\\|"
        "Ambient\\|AmbientAlgebra\\|"
        "AmbientDimension\\|AmbientModule\\|"
        "AmbientMonoid\\|AmbientSpace\\|"
        "AmbiguousForms\\|And\\|"
        "AnemicHeckeRing\\|Angle\\|"
        "Annihilator\\|AntisymmetricForms\\|"
        "Append\\|AppendBasePoint\\|"
        "AppendModule\\|Apply\\|"
        "Arccos\\|Arccosec\\|"
        "Arccot\\|Arcsec\\|"
        "Arcsin\\|Arctan\\|"
        "AreCollinear\\|AreIdentical\\|"
        "AreProportional\\|Arg\\|"
        "Argcosech\\|Argcosh\\|"
        "Argcoth\\|Argsech\\|"
        "Argsinh\\|Argtanh\\|"
        "Argument\\|ArithmeticGenus\\|"
        "ArithmeticVolume\\|ArrowWeights\\|"
        "Arrows\\|ArtinMap\\|"
        "ArtinSchreier\\|AssertAttribute\\|"
        "AssignNames\\|AssociatedNewSpace\\|"
        "AssociativeAlgebra\\|AtkinLehner\\|"
        "AtkinLehnerDecomposition\\|AtkinLehnerEigenspaceDimension\\|"
        "AtkinLehnerEigenvalue\\|AtkinLehnerInvolution\\|"
        "AtkinLehnerNewEigenspaceDimension\\|AtkinLehnerNewEigenspaceDimensions\\|"
        "AtkinLehnerPrimes\\|AtkinModularFunction\\|"
        "AtkinModularFunctionDatabase\\|AtkinModularPolynomial\\|"
        "Attach\\|AttachPackage\\|"
        "AttachSpec\\|AugmentCode\\|"
        "Augmentation\\|AugmentationIdeal\\|"
        "AugmentationMap\\|Aut\\|"
        "AutOrder\\|Automorphism\\|"
        "AutomorphismGroup\\|AutomorphismWorld\\|"
        "Automorphisms\\|AuxiliaryLevel\\|"
        "AxisMultiplicities\\|BCHBound\\|"
        "BCHCode\\|BachBound\\|"
        "BadPlaces\\|BadPrimes\\|"
        "BaerDerivation\\|BaerSubplane\\|"
        "Base\\|BaseAction\\|"
        "BaseChange\\|BaseChangeMatrix\\|"
        "BaseComponent\\|BaseCurve\\|"
        "BaseElement\\|BaseExtend\\|"
        "BaseField\\|BaseImage\\|"
        "BaseModule\\|BasePoint\\|"
        "BasePointSelect\\|BasePoints\\|"
        "BaseRing\\|BaseScheme\\|"
        "BaseSize\\|BaseSpace\\|"
        "BaseToSelfMap\\|BasicAlgebra\\|"
        "BasicOrbit\\|BasicOrbitLength\\|"
        "BasicOrbitLengths\\|BasicOrbits\\|"
        "BasicParameters\\|BasicStabilizer\\|"
        "Basis\\|BasisElement\\|"
        "BasisMatrix\\|BasisMinus\\|"
        "BasisPlus\\|BasisProduct\\|"
        "BasisProducts\\|BerlekampMassey\\|"
        "Bernoulli\\|BernoulliNumber\\|"
        "BesselFunction\\|BestApproximation\\|"
        "BetaFunction\\|Bicomponents\\|"
        "BinaryCodedForm\\|BinaryGenera\\|"
        "BinaryQuadraticForms\\|BinarySpinorGenera\\|"
        "BinaryStringMonoid\\|BinaryThetaSeries\\|"
        "BinomialPolynomial\\|BipartiteGraph\\|"
        "Bipartition\\|BitDecoding\\|"
        "BitEncoding\\|BitStringToInteger\\|"
        "BlackboxGroup\\|BlockLength\\|"
        "Blowup\\|Booleans\\|"
        "BorelSubgroup\\|BoundaryMap\\|"
        "BoundaryMaps\\|BraidGroup\\|"
        "BranchVertexPath\\|BrandtDecompositionDatabase\\|"
        "BrandtGroupoid\\|BrandtModule\\|"
        "BrandtModuleDatabase\\|BrauerCharacter\\|"
        "BravaisGroup\\|BurnsideMatrix\\|"
        "CRT\\|CambridgeMatrix\\|"
        "CanChangeUniverse\\|CanIdentifyGroup\\|"
        "CanonicalClass\\|CanonicalDedekind\\|"
        "CanonicalDegree\\|CanonicalDivisor\\|"
        "CanonicalGraph\\|CanonicalHeight\\|"
        "CanonicalInvolution\\|CanonicalInvolutionDatabase\\|"
        "CanonicalLength\\|CanonicalLift\\|"
        "CanonicalMap\\|CanonicalModularPolynomial\\|"
        "Capacity\\|CarmichaelLambda\\|"
        "CartanMatrix\\|CartesianPower\\|"
        "CartesianProduct\\|Cartier\\|"
        "Catalan\\|Category\\|"
        "CategoryList\\|CayleyGraph\\|"
        "CayleyGraphGroup\\|Ceiling\\|"
        "Center\\|CenterDensity\\|"
        "CentralExtension\\|CentralExtensions\\|"
        "CentralisingMatrix\\|Centralizer\\|"
        "Chabauty\\|ChainMap\\|"
        "ChangGraphs\\|ChangeBase\\|"
        "ChangeDirectory\\|ChangeOrder\\|"
        "ChangePrecision\\|ChangeRing\\|"
        "ChangeSign\\|ChangeSupport\\|"
        "ChangeUniverse\\|CharSeqs\\|"
        "Character\\|CharacterDegrees\\|"
        "CharacterRing\\|CharacterTable\\|"
        "CharacterToModular\\|Characteristic\\|"
        "CharacteristicPolynomial\\|Characters\\|"
        "ChebyshevFirst\\|ChebyshevSecond\\|"
        "ChebyshevT\\|ChebyshevU\\|"
        "CheckPolynomial\\|CheckTorsion\\|"
        "ChevalleyGroup\\|ChiefFactors\\|"
        "ChiefSeries\\|ChienChoyCode\\|"
        "ChineseRemainderTheorem\\|Cholesky\\|"
        "Choose\\|ChromaticIndex\\|"
        "ChromaticNumber\\|CipherType\\|"
        "Class\\|ClassAction\\|"
        "ClassFunctionSpace\\|ClassGroup\\|"
        "ClassGroupChecks\\|ClassGroupPRank\\|"
        "ClassImage\\|ClassMap\\|"
        "ClassMatrix\\|ClassNumber\\|"
        "ClassNumberDatabase\\|ClassNumbers\\|"
        "ClassPolynomial\\|ClassPolynomialDatabase\\|"
        "ClassPolynomials\\|ClassRelations\\|"
        "ClassSystems\\|ClassTwo\\|"
        "ClassUnion\\|Classes\\|"
        "ClassesExtend\\|ClassesTF\\|"
        "ClassicalForms\\|ClassicalModularPolynomial\\|"
        "ClassicalPeriod\\|ClassicalType\\|"
        "ClebschGraph\\|ClebschInvariants\\|"
        "ClebschToIgusaClebsch\\|CliffordAlgebra\\|"
        "CliqueNumber\\|CloseSmallGroupDatabase\\|"
        "CloseVectors\\|CloseVectorsMatrix\\|"
        "ClosestVectors\\|ClosureGraph\\|"
        "Cluster\\|Code\\|"
        "CodeComplement\\|CodeEntry\\|"
        "CodeGenerators\\|CodeToString\\|"
        "Codimension\\|Codomain\\|"
        "CodomainAlphabet\\|Coefficient\\|"
        "CoefficientField\\|CoefficientMap\\|"
        "CoefficientRing\\|CoefficientSpace\\|"
        "Coefficients\\|Coercion\\|"
        "Coincidence\\|CoincidenceDeficit\\|"
        "CoincidenceIndex\\|Cokernel\\|"
        "Collect\\|CollectRelations\\|"
        "CollineationGroup\\|ColonIdeal\\|"
        "Column\\|ColumnLength\\|"
        "ColumnSkewLength\\|ColumnSubmatrix\\|"
        "ColumnWeight\\|ColumnWeights\\|"
        "ColumnWord\\|Columns\\|"
        "CommonComponent\\|CommonOverfield\\|"
        "CommonZeros\\|Commutator\\|"
        "CommutatorGroup\\|CommutatorIdeal\\|"
        "CommutatorModule\\|CommutatorSubgroup\\|"
        "CompanionMatrix\\|Complement\\|"
        "ComplementBasis\\|ComplementVectors\\|"
        "Complements\\|Complete\\|"
        "CompleteClassGroup\\|CompleteDigraph\\|"
        "CompleteGraph\\|CompleteKArc\\|"
        "CompleteTupleList\\|CompleteUnion\\|"
        "Completion\\|Complex\\|"
        "ComplexConjugate\\|ComplexEmbeddings\\|"
        "ComplexField\\|ComplexObj\\|"
        "ComplexToPolar\\|ComplexValue\\|"
        "Component\\|ComponentProduct\\|"
        "Components\\|ComposeQuotients\\|"
        "Composite\\|CompositeFields\\|"
        "Composition\\|CompositionFactors\\|"
        "CompositionSeries\\|ConcatenatedCode\\|"
        "ConditionedGroup\\|Conductor\\|"
        "ConductorRange\\|CongruenceGroup\\|"
        "CongruenceIndices\\|CongruenceModulus\\|"
        "CongruenceSubgroup\\|Conic\\|"
        "ConjugacyClasses\\|ConjugacyClassesGL\\|"
        "Conjugate\\|ConjugatePartition\\|"
        "Conjugates\\|ConjugatingElement\\|"
        "Connect\\|ConnectionNumber\\|"
        "Consistency\\|ConstantField\\|"
        "ConstantMap\\|ConstantTerm\\|"
        "ConstantWords\\|Constituent\\|"
        "Constituents\\|Constraint\\|"
        "ContainsQuadrangle\\|Content\\|"
        "ContinuedFraction\\|Contract\\|"
        "Contraction\\|Convergents\\|"
        "Converse\\|Convolution\\|"
        "ConwayPolynomial\\|Coordelt\\|"
        "Coordinate\\|CoordinateLattice\\|"
        "CoordinateRing\\|CoordinateSpace\\|"
        "CoordinateVector\\|Coordinates\\|"
        "Coppersmith\\|CoprimeBasis\\|"
        "CordaroWagnerCode\\|Core\\|"
        "CorrelationGroup\\|Cos\\|"
        "Cosec\\|Cosech\\|"
        "CosetAction\\|CosetGeometry\\|"
        "CosetImage\\|CosetKernel\\|"
        "CosetLeaders\\|CosetSatisfying\\|"
        "CosetSpace\\|CosetTable\\|"
        "CosetsSatisfying\\|Cosh\\|"
        "Cot\\|Coth\\|"
        "Covalence\\|Covariant\\|"
        "CoveringMonoid\\|CoveringRadius\\|"
        "CoveringStructure\\|Cputime\\|"
        "CrackCharacter\\|CreateCycleFile\\|"
        "CreateICQAlias\\|CreateLieGroup\\|"
        "CreateRelshp\\|CremonaDatabase\\|"
        "CremonaDatabase\\|CremonaReference\\|"
        "CrossCorrelation\\|Cryptosystem\\|"
        "Cunningham\\|Current\\|"
        "CurrentLabel\\|Curve\\|"
        "CurveDivisor\\|CurvePlace\\|"
        "Cusp\\|CuspForms\\|"
        "CuspWidth\\|CuspidalSubgroup\\|"
        "CuspidalSubspace\\|Cusps\\|"
        "CutVertices\\|Cycle\\|"
        "CycleCount\\|CycleDecomposition\\|"
        "CycleStructure\\|CyclicCode\\|"
        "CyclicGroup\\|CyclicSubgroups\\|"
        "CyclotomicField\\|CyclotomicOrder\\|"
        "CyclotomyModule\\|Darstellungsgruppe\\|"
        "Data\\|DatabasePrecision\\|"
        "DatabasePrecision\\|DatabaseType\\|"
        "DatabaseType\\|DawsonIntegral\\|"
        "Decimation\\|Deciphering\\|"
        "Decode\\|DecodeAgGroup\\|"
        "DecodeML\\|Decoding\\|"
        "Decomp\\|DecomposeVector\\|"
        "Decomposition\\|DecompositionField\\|"
        "DecompositionGroup\\|DecompositionType\\|"
        "Decycle\\|DedekindEta\\|"
        "DeepHoles\\|DefinedInDegrees\\|"
        "DefiningIdeal\\|DefiningMap\\|"
        "DefiningMaps\\|DefiningModule\\|"
        "DefiningPoints\\|DefiningPolynomial\\|"
        "DefiningRelations\\|DefiningSpace\\|"
        "DefinitionSets\\|DegeneracyMap\\|"
        "DegeneracyMatrix\\|Degree\\|"
        "DegreeReduction\\|DegreeSequence\\|"
        "Degrees\\|DeleteAttributes\\|"
        "DeleteCollector\\|DeleteGenerator\\|"
        "DeleteLabel\\|DeleteLabels\\|"
        "DeleteProcess\\|DeleteProcessDown\\|"
        "DeleteRelation\\|Delta\\|"
        "Denominator\\|DenseED\\|"
        "Density\\|Depth\\|"
        "Derivative\\|DerivedGroup\\|"
        "DerivedLength\\|DerivedSeries\\|"
        "DerivedSubgroup\\|Descendants\\|"
        "Design\\|Detach\\|"
        "DetachPackage\\|DetachSpec\\|"
        "Determinant\\|Determinise\\|"
        "Determinize\\|Development\\|"
        "DiagonalForm\\|DiagonalJoin\\|"
        "DiagonalMatrix\\|DiagonalSum\\|"
        "Diagonalization\\|Diagram\\|"
        "Diameter\\|DiameterPath\\|"
        "DickmanRho\\|DicksonFirst\\|"
        "DicksonSecond\\|Difference\\|"
        "DifferenceSet\\|Different\\|"
        "DifferentDivisor\\|Differential\\|"
        "DifferentialBasis\\|DifferentialSpace\\|"
        "Differentiation\\|DihedralGroup\\|"
        "Dilog\\|Dimension\\|"
        "DimensionCuspForms\\|DimensionOfCenter\\|"
        "DimensionOfHom\\|DimensionsOfTerms\\|"
        "DirectProduct\\|DirectSum\\|"
        "DirichletCharacter\\|DirichletGroup\\|"
        "Disconnect\\|Discriminant\\|"
        "DisownChildren\\|Display\\|"
        "DisplayPolyMap\\|DisplayPolygons\\|"
        "Distance\\|DistanceMatrix\\|"
        "DistancePartition\\|Distinct\\|"
        "DistinctExtensions\\|DistinguishedRoot\\|"
        "DivisionFunction\\|DivisionPoints\\|"
        "DivisionPolynomial\\|DivisionPsi\\|"
        "DivisionSequence\\|Divisor\\|"
        "DivisorBasis\\|DivisorGroup\\|"
        "DivisorIdeal\\|DivisorMap\\|"
        "DivisorOfDegreeOne\\|DivisorSigma\\|"
        "DivisorToPoint\\|Divisors\\|"
        "Dixon\\|Domain\\|"
        "DomainAlphabet\\|DomainIdeal\\|"
        "Double\\|DoubleCoset\\|"
        "DoubleCosets\\|Dual\\|"
        "DualAtkinLehner\\|DualBasisLattice\\|"
        "DualFan\\|DualGraphCanonical\\|"
        "DualHeckeOperator\\|DualIdeal\\|"
        "DualLattice\\|DualModularSymbol\\|"
        "DualPartition\\|DualQuotient\\|"
        "DualRepresentation\\|DualStarInvolution\\|"
        "DualVectorSpace\\|ECM\\|"
        "EchelonBasis\\|EchelonForm\\|"
        "EchelonSeries\\|EcheloniseWord\\|"
        "EchelonizeWord\\|EdgeConnectivity\\|"
        "EdgeDeterminant\\|EdgeGroup\\|"
        "EdgeIndices\\|EdgeLabel\\|"
        "EdgeLabels\\|EdgeMultiplicity\\|"
        "EdgeSeparator\\|EdgeSet\\|"
        "EdgeUnion\\|Edges\\|"
        "EichlerOrder\\|Eigenspace\\|"
        "Eigenvalues\\|Eisenstein\\|"
        "EisensteinData\\|EisensteinIntegers\\|"
        "EisensteinSeries\\|EisensteinSubspace\\|"
        "ElementFromCFE\\|ElementSequence\\|"
        "ElementSet\\|ElementToList\\|"
        "ElementToSequence\\|ElementToString\\|"
        "ElementType\\|ElementaryDivisors\\|"
        "Elements\\|EliasBound\\|"
        "Elim\\|Eliminate\\|"
        "Elimination\\|EliminationIdeal\\|"
        "EllipticCurve\\|EllipticCurveDatabase\\|"
        "EllipticCurves\\|EllipticFactors\\|"
        "EllipticInvariants\\|EllipticLogarithm\\|"
        "EllipticPeriods\\|EllipticPoints\\|"
        "Eltseq\\|Embed\\|"
        "Embedding\\|EmbeddingMap\\|"
        "EmbeddingMatrix\\|EmbeddingSpace\\|"
        "EmptySubscheme\\|Enciphering\\|"
        "Encoding\\|End\\|"
        "EndomorphismRing\\|Endomorphisms\\|"
        "EnriquesForm\\|Entropy\\|"
        "Entry\\|EqualizeDegrees\\|"
        "EquationOrder\\|EquitablePartition\\|"
        "EquivalentPoint\\|ErrorFunction\\|"
        "EstimateOrbit\\|Eta\\|"
        "EuclideanNorm\\|EuclideanWeight\\|"
        "EulerFactor\\|EulerFactorModChar\\|"
        "EulerGamma\\|EulerPhi\\|"
        "EulerPhiInverse\\|Evaluate\\|"
        "EvaluateAt\\|EvaluateClassGroup\\|"
        "EvaluatePolynomial\\|EvenWeightCode\\|"
        "Exact\\|ExactConstantField\\|"
        "ExactExtension\\|ExactQuotient\\|"
        "ExactValue\\|Exclude\\|"
        "ExcludedConjugate\\|ExcludedConjugates\\|"
        "ExistsGroupData\\|ExistsModularCurveDatabase\\|"
        "Exp\\|Expand\\|"
        "ExpandToPrecision\\|ExplicitCoset\\|"
        "Exponent\\|ExponentLaw\\|"
        "ExponentSum\\|Exponents\\|"
        "ExpurgateCode\\|ExtGenerators\\|"
        "Extend\\|ExtendBasicOrbit\\|"
        "ExtendBasis\\|ExtendCode\\|"
        "ExtendCoding\\|ExtendField\\|"
        "ExtendModule\\|ExtendedRing\\|"
        "Extension\\|ExtensionClasses\\|"
        "ExtensionExponents\\|ExtensionNumbers\\|"
        "ExtensionPrimes\\|ExtensionProcess\\|"
        "ExtensionsOfModule\\|Exterior\\|"
        "ExteriorAlgebra\\|ExteriorModule\\|"
        "ExteriorPower\\|ExteriorProduct\\|"
        "ExteriorSquare\\|ExternalLines\\|"
        "ExtraSpecialAction\\|ExtraSpecialBasis\\|"
        "ExtraSpecialGroup\\|ExtractBlock\\|"
        "ExtractBlockRange\\|ExtractGenerators\\|"
        "ExtractGroup\\|Face\\|"
        "FaceFunction\\|Faces\\|"
        "FacesContaining\\|Factor\\|"
        "FactorBasis\\|FactorBasisCreate\\|"
        "FactoredEulerPhi\\|FactoredIndex\\|"
        "FactoredModulus\\|FactoredOrder\\|"
        "Factorization\\|FareySymbol\\|"
        "Field\\|FieldOfFractions\\|"
        "FilterVector\\|FindDependencies\\|"
        "FindGenerators\\|FindPerm\\|"
        "FindPowerSeries\\|FindRelations\\|"
        "FindWord\\|FiniteAffinePlane\\|"
        "FiniteField\\|FiniteIsogenyGraph\\|"
        "FireCode\\|FirstCohomology\\|"
        "FirstIndexOfColumn\\|FirstIndexOfRow\\|"
        "FittingGroup\\|FittingLength\\|"
        "FittingSeries\\|FittingSubgroup\\|"
        "Fix\\|FixedArc\\|"
        "FixedField\\|FixedGroup\\|"
        "FixedPoints\\|Flat\\|"
        "FlatProduct\\|FlattenPrecision\\|"
        "Fld\\|FldAbClassField\\|"
        "FldAbCreate\\|FldAbDecompose\\|"
        "FldAbGenerator\\|FldAbGeneratorCalc\\|"
        "FldAbIntersect\\|FldAbReduce\\|"
        "FletcherNumber\\|Floor\\|"
        "Flush\\|ForceSingularities\\|"
        "FormType\\|FormalEmbedding\\|"
        "FormalIsogeny\\|FormalRelation\\|"
        "FormalSet\\|Format\\|"
        "FrattiniSubgroup\\|FreeAbelianGroup\\|"
        "FreeAbelianMonoid\\|FreeAlgebra\\|"
        "FreeGroup\\|FreeMonoid\\|"
        "FreeNilpotentGroup\\|FreeProduct\\|"
        "FreeRepresentation\\|FreeResolution\\|"
        "FreeSemigroup\\|Frequency\\|"
        "Frobenius\\|FrobeniusMap\\|"
        "Function\\|FunctionDegree\\|"
        "FunctionField\\|FunctionFieldPatch\\|"
        "FunctionFieldPlace\\|FunctionRelations\\|"
        "FundamentalDomain\\|FundamentalElement\\|"
        "FundamentalKernel\\|FundamentalUnit\\|"
        "FundamentalUnits\\|GCD\\|"
        "GF\\|GHom\\|"
        "GModule\\|GModuleAction\\|"
        "GModulePrimes\\|GSet\\|"
        "GSetFromIndexed\\|GabidulinCode\\|"
        "GalFactGroup\\|GalFactGroupQuot\\|"
        "GaloisClosure\\|GaloisConjugate\\|"
        "GaloisConjugates\\|GaloisFactorGroup\\|"
        "GaloisField\\|GaloisGroup\\|"
        "GaloisImage\\|GaloisOrbit\\|"
        "GaloisRing\\|Gamma\\|"
        "Gamma0\\|Gamma1\\|"
        "Gamma2\\|Gamma3\\|"
        "Gamma4\\|GammaUpper0\\|"
        "GammaUpper1\\|GapNumbers\\|"
        "GaussianIntegers\\|Gcd\\|"
        "GeneralFindPoint\\|GeneralLinearGroup\\|"
        "GenerateGraphs\\|GeneratepGroups\\|"
        "GeneratingWords\\|Generator\\|"
        "GeneratorMatrix\\|GeneratorNumber\\|"
        "GeneratorString\\|GeneratorStructure\\|"
        "Generators\\|GeneratorsSequence\\|"
        "Generic\\|GenericGenus\\|"
        "GenericGroup\\|GenericHom\\|"
        "GenericPoint\\|GenericPolynomial\\|"
        "GenericToFF\\|GenericToFOF\\|"
        "Genus\\|GenusContribution\\|"
        "GenusField\\|Geodesic\\|"
        "GeometricGenus\\|GeometricSupport\\|"
        "Germ\\|GetAttributes\\|"
        "GetChild\\|GetChildren\\|"
        "GetForceCFP\\|GetI\\|"
        "GetLibraries\\|GetLibraryRoot\\|"
        "GetModule\\|GetModules\\|"
        "GetParent\\|GetPath\\|"
        "GetPresentation\\|GetPrimes\\|"
        "GetPrintLevel\\|GetPrompt\\|"
        "GetQuotient\\|GetRandomState\\|"
        "GetShareRoot\\|GetVerbose\\|"
        "Getc\\|Gets\\|"
        "GewirtzGraph\\|Girth\\|"
        "GirthCycle\\|GlobalUnitGroup\\|"
        "GoethalsCode\\|GolayCode\\|"
        "GoodBasePoints\\|GoppaCode\\|"
        "GradedRingData\\|GradedRingDatabase\\|"
        "GradedRingDatabase\\|GradientVector\\|"
        "GradientVectors\\|Gradings\\|"
        "GramMatrix\\|GramSchmidtReduce\\|"
        "Graph\\|GraphDatabase\\|"
        "GraphDatabase\\|Graphs\\|"
        "GrayMap\\|GrayMapImage\\|"
        "GreatestLowerBound\\|GriesmerBound\\|"
        "Groebner\\|GroebnerBasis\\|"
        "GroebnerRelations\\|GroebnerWalk\\|"
        "GrossZagierData\\|GrossZagierLevels\\|"
        "GroundField\\|Group\\|"
        "Group2NilData\\|Group3NilData\\|"
        "GroupAlgebra\\|GroupCurve\\|"
        "GroupData\\|GroupGenerators\\|"
        "GroupHom\\|GroupToMonoid\\|"
        "GrowthFunction\\|HadamardGraph\\|"
        "HadamardNormalize\\|HadamardRowDesign\\|"
        "HallSubgroup\\|HammingCode\\|"
        "HammingWeightEnumerator\\|HarmonicNumber\\|"
        "HasAttribute\\|HasCentreType\\|"
        "HasClique\\|HasComplement\\|"
        "HasComputableLCS\\|HasCoordinates\\|"
        "HasCurve\\|HasDecomposition\\|"
        "HasDefinedTerm\\|HasDenseRep\\|"
        "HasDenseRepOnly\\|HasFactorization\\|"
        "HasFiniteAQ\\|HasFiniteOrder\\|"
        "HasGCD\\|HasGroebnerBasis\\|"
        "HasImageIn\\|HasIrregularFibres\\|"
        "HasKnownInverse\\|HasOddDegreeModel\\|"
        "HasOrder\\|HasOrderDividing\\|"
        "HasOutputFile\\|HasPRoot\\|"
        "HasParallelClass\\|HasParallelism\\|"
        "HasPlace\\|HasPoint\\|"
        "HasPolynomial\\|HasPreimage\\|"
        "HasRationalPoint\\|HasReducedFibres\\|"
        "HasReducedPoint\\|HasRepresentation\\|"
        "HasResidualRoot\\|HasResolution\\|"
        "HasResultant\\|HasRightInverse\\|"
        "HasRoot\\|HasSignature\\|"
        "HasSparseRep\\|HasSparseRepOnly\\|"
        "HasSupplement\\|HasValidCosetTable\\|"
        "HasValidIndex\\|HasseInvariant\\|"
        "HasseWittInvariant\\|Hauteur\\|"
        "HauteurNaive\\|HeckeAlgebra\\|"
        "HeckeAlgebraFields\\|HeckeAlgebraZBasis\\|"
        "HeckeBound\\|HeckeCorrespondenceDatabase\\|"
        "HeckeFieldSpan\\|HeckeModule\\|"
        "HeckeOperator\\|HeckePolynomial\\|"
        "HeckeRing\\|HeckeSpan\\|"
        "HeckeTrace\\|Height\\|"
        "HeightConstant\\|HeightPairing\\|"
        "HeilbronnCremona\\|HeilbronnMerel\\|"
        "HenselLift\\|HermiteForm\\|"
        "HermiteNumber\\|HermitePolynomial\\|"
        "HermitianCode\\|HermitianCurve\\|"
        "HermitianFunctionField\\|HermitianModule\\|"
        "HessenbergForm\\|HessianMatrix\\|"
        "HessianSubscheme\\|HilbertClassField\\|"
        "HilbertClassPolynomial\\|HilbertForm\\|"
        "HilbertFunction\\|HilbertMatrix\\|"
        "HilbertNumerator\\|HilbertPolynomial\\|"
        "HilbertSeries\\|HilbertSymbol\\|"
        "Hint\\|HirschNumber\\|"
        "Holes\\|Hom\\|"
        "Homogenization\\|Homology\\|"
        "Homomorphism\\|Homomorphisms\\|"
        "HookLength\\|HorizontalFunction\\|"
        "HorizontalJoin\\|HorizontalVertices\\|"
        "Hypercenter\\|Hypercentre\\|"
        "HyperellipticCurve\\|HyperellipticCurveFromIgusaClebsch\\|"
        "HyperellipticCurveOfGenus\\|HyperellipticIntegral\\|"
        "HyperellipticInvolution\\|HyperellipticPolynomials\\|"
        "HypergeometricSeries\\|HypergeometricU\\|"
        "ICQPublicKey\\|ICQRead\\|"
        "ICQSend\\|ICQUsers\\|"
        "IL\\|ILO\\|"
        "IMT\\|IRO\\|"
        "ISA\\|Id\\|"
        "Ideal\\|IdealQuotient\\|"
        "Idealiser\\|Idealizer\\|"
        "Ideals\\|Idempotent\\|"
        "Identifier\\|IdentifyGroup\\|"
        "IdentifyLieType\\|IdentifySimple\\|"
        "Identity\\|IdentityIsogeny\\|"
        "IdentityMap\\|IdentityMatrix\\|"
        "IgusaClebschInvariants\\|IgusaClebschToClebsch\\|"
        "IgusaInvariants\\|IharaBound\\|"
        "Im\\|Image\\|"
        "ImageSystem\\|ImageWithBasis\\|"
        "Images\\|Imaginary\\|"
        "ImplicitFunction\\|Implicitization\\|"
        "ImprimitiveAction\\|InDegree\\|"
        "InEdge\\|InNeighbors\\|"
        "InNeighbours\\|IncidenceDigraph\\|"
        "IncidenceGeometry\\|IncidenceGraph\\|"
        "IncidenceMatrix\\|IncidenceStructure\\|"
        "IncidentEdges\\|Include\\|"
        "InclusionMap\\|IndependenceNumber\\|"
        "IndependentUnits\\|Index\\|"
        "IndexFormEquation\\|IndexOfPartition\\|"
        "IndexOfSpeciality\\|IndexedCoset\\|"
        "IndexedSetToSet\\|Indices\\|"
        "InduceExtend\\|InducedMap\\|"
        "InducedPermutation\\|Induction\\|"
        "InertiaDegree\\|InertiaElement\\|"
        "InertiaField\\|InertiaGroup\\|"
        "InertiaRing\\|InertialPolynomial\\|"
        "InertseqpAdic\\|Infimum\\|"
        "InfinitePlaces\\|InfinitePolynomial\\|"
        "InfiniteSum\\|Infinity\\|"
        "InflectionPoints\\|InformationRate\\|"
        "InformationSet\\|InformationSpace\\|"
        "InitialFactors\\|InitialForm\\|"
        "InitialStates\\|Initialize\\|"
        "InitializeBase\\|Injection\\|"
        "Injections\\|InjectiveHull\\|"
        "InjectiveModule\\|InnerFaces\\|"
        "InnerGenerators\\|InnerProduct\\|"
        "InnerProductMatrix\\|InnerShape\\|"
        "InnerVertices\\|Insert\\|"
        "InsertBlock\\|InsertVertex\\|"
        "IntNInf\\|IntegerRing\\|"
        "IntegerToBitString\\|IntegerToHexString\\|"
        "IntegerToSequence\\|IntegerToString\\|"
        "Integers\\|Integral\\|"
        "IntegralBasis\\|IntegralBasisMinus\\|"
        "IntegralBasisPlus\\|IntegralClosure\\|"
        "IntegralGroup\\|IntegralKernel\\|"
        "IntegralMapping\\|IntegralModel\\|"
        "IntegralOrder\\|IntegralPoints\\|"
        "IntegralSplit\\|Interior\\|"
        "Interpolation\\|IntersectKernels\\|"
        "Intersection\\|IntersectionArray\\|"
        "IntersectionGroup\\|IntersectionMatrix\\|"
        "IntersectionNumber\\|IntersectionPoints\\|"
        "Intrinsics\\|InvHom\\|"
        "InvariantBasis\\|InvariantFactors\\|"
        "InvariantForms\\|InvariantRing\\|"
        "Invariants\\|InvariantsOfDegree\\|"
        "Inverse\\|InverseDefiningPolynomials\\|"
        "InverseJeuDeTaquin\\|InverseKey\\|"
        "InverseKrawchouk\\|InverseMod\\|"
        "InverseRowInsert\\|InverseSqrt\\|"
        "InverseWordMap\\|Inverses\\|"
        "Involution\\|IrreducibleComponents\\|"
        "IrreducibleModule\\|IrreducibleModules\\|"
        "IrregularValues\\|IrregularVertices\\|"
        "IsAbelian\\|IsAbelianStep\\|"
        "IsAbsoluteField\\|IsAbsoluteOrder\\|"
        "IsAbsolutelyIrreducible\\|IsAdditive\\|"
        "IsAffine\\|IsAffineLinear\\|"
        "IsAlternating\\|IsAltsym\\|"
        "IsAmbient\\|IsAmbientFFElt\\|"
        "IsAmbientFunction\\|IsAmbientSpace\\|"
        "IsAnemic\\|IsArc\\|"
        "IsAssociative\\|IsAutomatic\\|"
        "IsAutomorphism\\|IsBalanced\\|"
        "IsBasePointFree\\|IsBiconnected\\|"
        "IsBijective\\|IsBinary\\|"
        "IsBipartite\\|IsBlock\\|"
        "IsBlockTransitive\\|IsBoundary\\|"
        "IsCanonical\\|IsCentral\\|"
        "IsChainMap\\|IsCharacter\\|"
        "IsCipherText\\|IsCluster\\|"
        "IsCoercible\\|IsCohenMacaulay\\|"
        "IsCollinear\\|IsCommutative\\|"
        "IsComplete\\|IsComponent\\|"
        "IsConcurrent\\|IsConditioned\\|"
        "IsConfluent\\|IsCongruence\\|"
        "IsConic\\|IsConjugate\\|"
        "IsConnected\\|IsConnectedFibre\\|"
        "IsConsistent\\|IsConstant\\|"
        "IsConway\\|IsCurve\\|"
        "IsCurveFFElt\\|IsCusp\\|"
        "IsCuspidal\\|IsCuspidalNewform\\|"
        "IsCyclic\\|IsDecomposable\\|"
        "IsDeficient\\|IsDefined\\|"
        "IsDefinite\\|IsDegenerate\\|"
        "IsDesarguesian\\|IsDesign\\|"
        "IsDiagonal\\|IsDifferenceSet\\|"
        "IsDirectSummand\\|IsDirected\\|"
        "IsDisjoint\\|IsDistanceRegular\\|"
        "IsDivisibleBy\\|IsDivisionRing\\|"
        "IsDomain\\|IsDominant\\|"
        "IsDoublePoint\\|IsDoublyEven\\|"
        "IsEdgeTransitive\\|IsEffective\\|"
        "IsEisenstein\\|IsEisensteinSeries\\|"
        "IsEllipticCurve\\|IsElmsym\\|"
        "IsEmpty\\|IsEmptyWord\\|"
        "IsEndomorphism\\|IsEof\\|"
        "IsEqual\\|IsEquationOrder\\|"
        "IsEquidistant\\|IsEquitable\\|"
        "IsEquivalent\\|IsEuclideanDomain\\|"
        "IsEuclideanRing\\|IsEulerian\\|"
        "IsEven\\|IsExact\\|"
        "IsExactlyDivisible\\|IsExceptionalUnit\\|"
        "IsExtension\\|IsExtraSpecial\\|"
        "IsFTGeometry\\|IsFace\\|"
        "IsFaithful\\|IsField\\|"
        "IsFieldType\\|IsFinite\\|"
        "IsFiniteIndex\\|IsFiniteOrder\\|"
        "IsFirm\\|IsFlex\\|"
        "IsForest\\|IsFree\\|"
        "IsFrobenius\\|IsFull\\|"
        "IsGL2Equivalent\\|IsGLattice\\|"
        "IsGamma\\|IsGamma0\\|"
        "IsGamma1\\|IsGammaUpper0\\|"
        "IsGammaUpper1\\|IsGenus\\|"
        "IsGenusComputable\\|IsGerm\\|"
        "IsGlobal\\|IsGlobalUnit\\|"
        "IsGraph\\|IsGroebner\\|"
        "IsHadamard\\|IsHexadecimal\\|"
        "IsHolzerReduced\\|IsHomeomorphic\\|"
        "IsHomogeneous\\|IsHomogenous\\|"
        "IsHomomorphism\\|IsHomsym\\|"
        "IsHyperellipticCurve\\|IsHyperellipticCurveOfGenus\\|"
        "IsHyperellipticWeierstrass\\|IsHypersurface\\|"
        "IsId\\|IsIdeal\\|"
        "IsIdempotent\\|IsIdentical\\|"
        "IsIdentity\\|IsIdentityProduct\\|"
        "IsInImage\\|IsInRadical\\|"
        "IsInSmallGroupDatabase\\|IsInWorld\\|"
        "IsIndecomposable\\|IsIndefinite\\|"
        "IsIndependent\\|IsInert\\|"
        "IsInertial\\|IsInfinite\\|"
        "IsInflectionPoint\\|IsInjective\\|"
        "IsInner\\|IsIntegral\\|"
        "IsIntegralDomain\\|IsIntegralModel\\|"
        "IsInterior\\|IsIntrinsic\\|"
        "IsInvariant\\|IsInvertible\\|"
        "IsIrreducible\\|IsIsogenous\\|"
        "IsIsometric\\|IsIsomorphic\\|"
        "IsIsomorphicDRK\\|IsIsomorphism\\|"
        "IsIsotropic\\|IsJacobianPencil\\|"
        "IsJordan\\|IsKEdgeConnected\\|"
        "IsKVertexConnected\\|IsKnuthEquivalent\\|"
        "IsLE\\|IsLabelled\\|"
        "IsLabelledEdge\\|IsLabelledVertex\\|"
        "IsLe\\|IsLeftIdeal\\|"
        "IsLeftIsomorphic\\|IsLehmerCode\\|"
        "IsLessThan\\|IsLie\\|"
        "IsLineRegular\\|IsLineTransitive\\|"
        "IsLinear\\|IsLinearGroup\\|"
        "IsLinearScheme\\|IsLinearSpace\\|"
        "IsLocalNorm\\|IsMatrixModule\\|"
        "IsMaximal\\|IsMemberBasicOrbit\\|"
        "IsMinimalModel\\|IsMinusOne\\|"
        "IsMinusQuotient\\|IsModular\\|"
        "IsModularCurve\\|IsModularForm\\|"
        "IsMonomial\\|IsMultiChar\\|"
        "IsMultiplicative\\|IsNearLinearSpace\\|"
        "IsNearlyPerfect\\|IsNegativeDefinite\\|"
        "IsNew\\|IsNewSubring\\|"
        "IsNewform\\|IsNewtonPolygonOf\\|"
        "IsNilpotent\\|IsNilpotentLuks\\|"
        "IsNode\\|IsNonSingular\\|"
        "IsNonsingular\\|IsNontrivial\\|"
        "IsNorm\\|IsNormal\\|"
        "IsNull\\|IsOctal\\|"
        "IsOddDegree\\|IsOnCurve\\|"
        "IsOne\\|IsOrbit\\|"
        "IsOrder\\|IsOrdered\\|"
        "IsOrdinary\\|IsOrdinaryProjective\\|"
        "IsOrdinarySingularity\\|IsOrthogonalGroup\\|"
        "IsOverSmallerField\\|IsPID\\|"
        "IsPIR\\|IsParallel\\|"
        "IsParallelClass\\|IsParallelism\\|"
        "IsPartialRoot\\|IsPartition\\|"
        "IsPartitionRefined\\|IsPath\\|"
        "IsPerfect\\|IsPlainText\\|"
        "IsPlanar\\|IsPlusQuotient\\|"
        "IsPoint\\|IsPointRegular\\|"
        "IsPointTransitive\\|IsPolygon\\|"
        "IsPolynomial\\|IsPositive\\|"
        "IsPositiveDefinite\\|IsPower\\|"
        "IsPowerLarge\\|IsPowsym\\|"
        "IsPrimary\\|IsPrime\\|"
        "IsPrimeCertificate\\|IsPrimeForm\\|"
        "IsPrimePower\\|IsPrimitive\\|"
        "IsPrincipal\\|IsPrincipalIdealDomain\\|"
        "IsPrincipalIdealRing\\|IsProbablyMaximal\\|"
        "IsProjective\\|IsProper\\|"
        "IsProperChainMap\\|IsProportional\\|"
        "IsPureOrder\\|IsQuadratic\\|"
        "IsQuadraticTwist\\|IsRC\\|"
        "IsRadical\\|IsRadix64\\|"
        "IsRamified\\|IsRationalCurve\\|"
        "IsRationalPoint\\|IsReal\\|"
        "IsReduced\\|IsRegular\\|"
        "IsRegularSequence\\|IsRepresentation\\|"
        "IsResolution\\|IsRightIdeal\\|"
        "IsRightIsomorphic\\|IsRingHomomorphism\\|"
        "IsRoot\\|IsRootedTree\\|"
        "IsSIntegral\\|IsSPrincipal\\|"
        "IsSUnit\\|IsSatisfied\\|"
        "IsScalar\\|IsSchur\\|"
        "IsSelfDual\\|IsSelfNormalising\\|"
        "IsSelfNormalizing\\|IsSelfOrthogonal\\|"
        "IsSemiDefinite\\|IsSemiLinear\\|"
        "IsSemiregular\\|IsSemisimple\\|"
        "IsSeparable\\|IsSeparating\\|"
        "IsShimuraSubgroup\\|IsSimilar\\|"
        "IsSimple\\|IsSimpleOrder\\|"
        "IsSimpleXXX\\|IsSimplifiedModel\\|"
        "IsSingular\\|IsSkew\\|"
        "IsSmooth\\|IsSoluble\\|"
        "IsSolubleSSS\\|IsSolvable\\|"
        "IsSolvableSSS\\|IsSpecial\\|"
        "IsSpinorGenus\\|IsSpinorNorm\\|"
        "IsSplit\\|IsSplittingField\\|"
        "IsSquare\\|IsSquarefree\\|"
        "IsStandard\\|IsSteiner\\|"
        "IsStronglyAG\\|IsSubfield\\|"
        "IsSubgraph\\|IsSubgroup\\|"
        "IsSubmodule\\|IsSubnormal\\|"
        "IsSubsequence\\|IsSubspace\\|"
        "IsSupersingular\\|IsSurjective\\|"
        "IsSymmetric\\|IsSymmetricSFA\\|"
        "IsSymplecticGroup\\|IsTamelyRamified\\|"
        "IsTangent\\|IsTensor\\|"
        "IsTensorInduced\\|IsThick\\|"
        "IsThin\\|IsTorsionUnit\\|"
        "IsTotallyRamified\\|IsTotallySplit\\|"
        "IsTransitive\\|IsTransverse\\|"
        "IsTree\\|IsTriconnected\\|"
        "IsTrivial\\|IsTwist\\|"
        "IsUFD\\|IsUndirected\\|"
        "IsUniform\\|IsUniqueFactorizationDomain\\|"
        "IsUnit\\|IsUnitWithPreimage\\|"
        "IsUnital\\|IsUnitary\\|"
        "IsUnitaryGroup\\|IsUnitaryRepresentation\\|"
        "IsUnivariate\\|IsUnramified\\|"
        "IsValid\\|IsVerbose\\|"
        "IsVertex\\|IsVertexTransitive\\|"
        "IsWeaklyAG\\|IsWeaklyConnected\\|"
        "IsWeaklyEqual\\|IsWeaklySelfDual\\|"
        "IsWeaklyZero\\|IsWeierstrassModel\\|"
        "IsWeierstrassPlace\\|IsWildlyRamified\\|"
        "IsZero\\|IsZeroComplex\\|"
        "IsZeroDimensional\\|IsZeroDivisor\\|"
        "IsZeroMap\\|IsZeroTerm\\|"
        "Isetseq\\|Isetset\\|"
        "Iso\\|IsogeniesAreEqual\\|"
        "Isogeny\\|IsogenyClass\\|"
        "IsogenyFromKernel\\|IsogenyGraph\\|"
        "IsogenyMapOmega\\|IsogenyMapPhi\\|"
        "IsogenyMapPhiMulti\\|IsogenyMapPsi\\|"
        "IsogenyMapPsiMulti\\|IsolGroup\\|"
        "IsolGroupDatabase\\|IsolGroupDatabase\\|"
        "IsolGuardian\\|IsolInfo\\|"
        "IsolProcess\\|IsolProcessGroup\\|"
        "IsolProcessInfo\\|IsolProcessIsEmpty\\|"
        "IsolProcessLabel\\|IsolProcessNext\\|"
        "IsolProcessOfField\\|IsolateRoots\\|"
        "Isometry\\|Isomorphism\\|"
        "IsomorphismData\\|IspIntegral\\|"
        "IspMaximal\\|IspMinimal\\|"
        "IspNormal\\|JBessel\\|"
        "JInvariants\\|Jacobi\\|"
        "JacobiTheta\\|JacobiThetaNullK\\|"
        "Jacobian\\|JacobianIdeal\\|"
        "JacobianMatrix\\|JacobianPoint\\|"
        "JacobianRelations\\|JacobianSequence\\|"
        "JacobsonRadical\\|JacobsonRadicalXXX\\|"
        "JenningsSeries\\|JeuDeTaquin\\|"
        "JordanForm\\|JustesenCode\\|"
        "Juxtaposition\\|K3Database\\|"
        "K3Database\\|K3Surface\\|"
        "K3SurfaceFromAFR\\|K3SurfaceRaw\\|"
        "K3SurfacesDB\\|KBessel\\|"
        "KBessel2\\|KCubeGraph\\|"
        "KMatrixSpace\\|KModule\\|"
        "KModuleWithBasis\\|KS3\\|"
        "KSpace\\|KSpaceWithBasis\\|"
        "KappaLattice\\|KeepAbelian\\|"
        "KeepElementary\\|KeepGeneratorOrder\\|"
        "KeepGroupAction\\|KeepPGroupWeights\\|"
        "KeepPrimePower\\|KeepSplit\\|"
        "KeepSplitAbelian\\|KerdockCode\\|"
        "Kernel\\|KernelMatrix\\|"
        "Kernels\\|KillingForm\\|"
        "KissingNumber\\|KleinQuartic\\|"
        "Knot\\|KnownIrreducibles\\|"
        "KodairaSymbol\\|KodairaSymbols\\|"
        "KostkaNumber\\|KrawchoukTransform\\|"
        "KroneckerCharacter\\|KroneckerProduct\\|"
        "KummerSurface\\|KummerSurfaceRaw\\|"
        "LCM\\|LFSRCryptosystem\\|"
        "LFSRSequence\\|LFSRStep\\|"
        "LHS\\|LLL\\|"
        "LLLBasisMatrix\\|LLLBlock\\|"
        "LLLGram\\|LLLGramMatrix\\|"
        "LLLH\\|LPProcess\\|"
        "LPolynomial\\|LRatio\\|"
        "LRatioOddPart\\|LSeries\\|"
        "LUB\\|Label\\|"
        "Labelling\\|Labels\\|"
        "LaguerrePolynomial\\|LaminatedLattice\\|"
        "Lanczos\\|LanguageSize\\|"
        "Laplace\\|LargestConductor\\|"
        "LargestDimension\\|LargestOrder\\|"
        "LastColumnEntry\\|LastIndexOfRow\\|"
        "Lattice\\|LatticeData\\|"
        "LatticeDatabase\\|LatticeDatabase\\|"
        "LatticeGenera\\|LatticeGeneraDatabase\\|"
        "LatticeName\\|LatticeWithBasis\\|"
        "LatticeWithGram\\|LaurentSeriesRing\\|"
        "LayerBoundary\\|LayerLength\\|"
        "LazySeries\\|Lcm\\|"
        "LeadingCoefficient\\|LeadingExponent\\|"
        "LeadingGenerator\\|LeadingMonomial\\|"
        "LeadingTerm\\|LeadingTotalDegree\\|"
        "LeastUpperBound\\|LeeDistance\\|"
        "LeeWeight\\|LeftAnnihilator\\|"
        "LeftConjugate\\|LeftCosetSpace\\|"
        "LeftDiv\\|LeftExactExtension\\|"
        "LeftGCD\\|LeftGcd\\|"
        "LeftIdeal\\|LeftIdealClasses\\|"
        "LeftIdeals\\|LeftIsomorphism\\|"
        "LeftLCM\\|LeftLcm\\|"
        "LeftNormalForm\\|LeftOrder\\|"
        "LeftZeroExtension\\|LegendreModel\\|"
        "LegendrePolynomial\\|LehmerCode\\|"
        "LehmerCodeToPerm\\|Length\\|"
        "LengthenCode\\|Lengths\\|"
        "Less\\|LessAttributes\\|"
        "LessSignatures\\|Level\\|"
        "Levels\\|LevenshteinBound\\|"
        "LexProduct\\|LieAlgebra\\|"
        "LieBracket\\|Lift\\|"
        "LiftCharacter\\|LiftCharacters\\|"
        "LiftHomomorphism\\|LiftIsogeny\\|"
        "LiftIsomorphism\\|LiftModule\\|"
        "LiftModules\\|LiftPoint\\|"
        "LiftSplitExtension\\|LimitExactLength\\|"
        "LimitExactSize\\|LimitExactWeight\\|"
        "LimitLength\\|LimitSize\\|"
        "LimitWeight\\|Line\\|"
        "LineAtInfinity\\|LineGraph\\|"
        "LineGroup\\|LineOrbits\\|"
        "LineSet\\|LinearCharacters\\|"
        "LinearCode\\|LinearComplexity\\|"
        "LinearDependence\\|LinearGraph\\|"
        "LinearProjection\\|LinearRelation\\|"
        "LinearSieve\\|LinearSieveLog\\|"
        "LinearSpace\\|LinearSys\\|"
        "LinearSystem\\|LinearSystemAtPhi\\|"
        "LinearSystemTrace\\|Lines\\|"
        "Linking\\|LinkingNumbers\\|"
        "List\\|ListAttributes\\|"
        "ListConflicts\\|ListDependencies\\|"
        "ListSignatures\\|ListSpec\\|"
        "LocalConjugates\\|LocalField\\|"
        "LocalGenera\\|LocalGenus\\|"
        "LocalGroebnerBasis\\|LocalHeight\\|"
        "LocalInformation\\|LocalMinimalModel\\|"
        "LocalMinimalModels\\|LocalRing\\|"
        "LocalUniformizer\\|Localization\\|"
        "Locseq\\|LocseqInert\\|"
        "Log\\|LogAGM\\|"
        "LogDerivative\\|LogGamma\\|"
        "LogIntegral\\|LogSuperSing\\|"
        "Logs\\|LongDivision\\|"
        "LowDimSubmodules\\|LowIndexProcess\\|"
        "LowIndexSubgroups\\|LowIndexSubmodules\\|"
        "LowerCentralSeries\\|LowerFaces\\|"
        "LowerVertices\\|MCPolynomials\\|"
        "MDSCode\\|MEANS\\|"
        "MGCD\\|MMatchingDimacs\\|"
        "MMatchingDimacsTRY\\|MSAut\\|"
        "MSQLetternonsplit\\|MSQLettersplit\\|"
        "MSQnonsplit\\|MSQnonsplitBase\\|"
        "MSQsplit\\|MSQsplitBase\\|"
        "MahlerMeasure\\|MakeDen\\|"
        "MakeIsSquare\\|MakeK3Database\\|"
        "MakeK3Database\\|MakeModCubes\\|"
        "MakePCMap\\|MakePlace\\|"
        "MakeResiduesSEA\\|MakeReverseMap\\|"
        "MakeSpliceDiagram\\|MakeTES\\|"
        "MakeType\\|ManhattanForm\\|"
        "ManinSymbol\\|MantissaExponent\\|"
        "MapToMatrix\\|Maps\\|"
        "MarkGroebner\\|Match\\|"
        "Matrix\\|MatrixAlgebra\\|"
        "MatrixBasisSpace\\|MatrixGroup\\|"
        "MatrixMinor\\|MatrixQuotient\\|"
        "MatrixRing\\|MatrixUnit\\|"
        "Max\\|MaxNorm\\|"
        "MaxOrthPCheck\\|MaxParabolics\\|"
        "MaxSubsTF2\\|MaxSubsTF3\\|"
        "Maxdeg\\|Maxim\\|"
        "MaximalAbelianSubfield\\|MaximalIdeals\\|"
        "MaximalLeftIdeals\\|MaximalOrder\\|"
        "MaximalOrderFinite\\|MaximalOvergroup\\|"
        "MaximalParabolics\\|MaximalPartition\\|"
        "MaximalRightIdeals\\|MaximalSolution\\|"
        "MaximalSubfields\\|MaximalSubgroups\\|"
        "MaximalSubgroupsH\\|MaximalSubgroupsTF\\|"
        "MaximalSubmodules\\|MaximalSuperGenus\\|"
        "Maximum\\|MaximumClique\\|"
        "MaximumDegree\\|MaximumFlow\\|"
        "MaximumInDegree\\|MaximumMatching\\|"
        "MaximumOutDegree\\|Maxindeg\\|"
        "Maxoutdeg\\|Meataxe\\|"
        "MergeFields\\|MergeFiles\\|"
        "MergeUnits\\|Min\\|"
        "Mindeg\\|MinimalBasis\\|"
        "MinimalBasisMatrix\\|MinimalBlocks\\|"
        "MinimalField\\|MinimalIdeals\\|"
        "MinimalInteger\\|MinimalLeftIdeals\\|"
        "MinimalModel\\|MinimalNormalSubgroup\\|"
        "MinimalNormalSubgroups\\|MinimalOverfields\\|"
        "MinimalOvergroup\\|MinimalOvergroups\\|"
        "MinimalParabolics\\|MinimalPartition\\|"
        "MinimalPartitions\\|MinimalPolynomial\\|"
        "MinimalPrimeComponents\\|MinimalRational\\|"
        "MinimalRightIdeals\\|MinimalSolution\\|"
        "MinimalSubmodule\\|MinimalSubmodules\\|"
        "Minimise\\|Minimize\\|"
        "Minimum\\|MinimumCut\\|"
        "MinimumDegree\\|MinimumDistance\\|"
        "MinimumInDegree\\|MinimumLeeDistance\\|"
        "MinimumLeeWeight\\|MinimumOutDegree\\|"
        "MinimumWeight\\|MinimumWeightRM\\|"
        "MinimumWord\\|MinimumWords\\|"
        "Minindeg\\|MinkowskiBound\\|"
        "MinkowskiLattice\\|MinkowskiReduction\\|"
        "MinkowskiSpace\\|MinorBoundary\\|"
        "MinorLength\\|Minors\\|"
        "Minoutdeg\\|Minus\\|"
        "MinusInfinity\\|MinusVolume\\|"
        "MixedCanonicalForm\\|Modexp\\|"
        "Modinv\\|ModularComposition\\|"
        "ModularCurve\\|ModularCurveDatabase\\|"
        "ModularCurveX0\\|ModularCurveX1\\|"
        "ModularCurves\\|ModularDegree\\|"
        "ModularForm\\|ModularForms\\|"
        "ModularFormsBasis\\|ModularFormsDatabase\\|"
        "ModularGenusX0\\|ModularKernel\\|"
        "ModularPolynomial\\|ModularPolynomialDatabase\\|"
        "ModularSolution\\|ModularSymbol\\|"
        "ModularSymbolEven\\|ModularSymbolOdd\\|"
        "ModularSymbols\\|Module\\|"
        "ModuleExtension\\|ModuleMap\\|"
        "ModuleMaps\\|Modules\\|"
        "Moduli\\|ModuliPoints\\|"
        "Modulus\\|MoebiusMu\\|"
        "MolienSeries\\|MonodromyPairing\\|"
        "MonodromyWeights\\|Monoid\\|"
        "Monomial\\|MonomialBasis\\|"
        "MonomialGroup\\|MonomialSubgroup\\|"
        "Monomials\\|MonomialsOfDegree\\|"
        "MordellWeilGroup\\|MordellWeilLattice\\|"
        "MordellWeilRank\\|MordellWeilRankBounds\\|"
        "Morphism\\|MultTableauIns\\|"
        "MultTableauJeu\\|MultiLinearSystem\\|"
        "MultiQuotientMaps\\|MultiRank\\|"
        "MultiSpaces\\|Multidegree\\|"
        "MultipartiteGraph\\|MultiplicatorRing\\|"
        "Multiplicities\\|Multiplicity\\|"
        "MultiplierMatrix\\|MultiplyColumn\\|"
        "MultiplyRow\\|MultisetToSet\\|"
        "Multisets\\|NaiveHeight\\|"
        "Name\\|NameSimple\\|"
        "Names\\|NaturalGroup\\|"
        "NearLinearSpace\\|NegationMap\\|"
        "Neighbor\\|NeighborClosure\\|"
        "Neighbors\\|Neighbour\\|"
        "NeighbourClosure\\|NeighbouringGerms\\|"
        "Neighbours\\|NewDecomposition\\|"
        "NewSubspace\\|Newform\\|"
        "Newforms\\|NewtonLift\\|"
        "NewtonLiftCycle\\|NewtonPolygon\\|"
        "NewtonSlopes\\|NextClass\\|"
        "NextElement\\|NextExtension\\|"
        "NextGraph\\|NextModule\\|"
        "NextRepresentation\\|NextSubgroup\\|"
        "NextVector\\|NextWord\\|"
        "Ngens\\|NilpotencyClass\\|"
        "NilpotentBoundary\\|NilpotentLength\\|"
        "NilpotentQuotient\\|NilpotentSection\\|"
        "NilpotentSubgroups\\|NoCommonComponent\\|"
        "NoetherForm\\|NonReducedFibres\\|"
        "NonZeroCoordinates\\|NonsolvableSubgroups\\|"
        "NonsplitCollector\\|NonsplitSection\\|"
        "Norm\\|NormAbs\\|"
        "NormDistribution\\|NormEquation\\|"
        "NormGroup\\|NormModule\\|"
        "NormResidueSymbol\\|NormSpace\\|"
        "NormalClosure\\|NormalComplements\\|"
        "NormalElement\\|NormalForm\\|"
        "NormalLattice\\|NormalSubgroups\\|"
        "NormalSubgroupsTF\\|Normalise\\|"
        "Normaliser\\|Normalize\\|"
        "Normalizer\\|Not\\|"
        "NpAdicField\\|NpAdicQuotientRing\\|"
        "NpAdicRing\\|Nrels\\|"
        "Nrows\\|Nsgens\\|"
        "NuclearRank\\|NullBasis\\|"
        "NullDimensions\\|NullGraph\\|"
        "NullSpace\\|Nullspace\\|"
        "NullspaceMatrix\\|Number\\|"
        "NumberField\\|NumberFieldSieve\\|"
        "NumberOfBlocks\\|NumberOfClasses\\|"
        "NumberOfColumns\\|NumberOfComponents\\|"
        "NumberOfCurves\\|NumberOfDivisors\\|"
        "NumberOfFaces\\|NumberOfGenerators\\|"
        "NumberOfGradings\\|NumberOfGraphs\\|"
        "NumberOfGroups\\|NumberOfInclusions\\|"
        "NumberOfLabels\\|NumberOfLattices\\|"
        "NumberOfLines\\|NumberOfNames\\|"
        "NumberOfPlaces\\|NumberOfPoints\\|"
        "NumberOfPunctures\\|NumberOfRelations\\|"
        "NumberOfRows\\|NumberOfSkewRows\\|"
        "NumberOfStrings\\|NumberOfVariables\\|"
        "NumberOfWords\\|NumberingMap\\|"
        "Numelt\\|Numerator\\|"
        "NumericalRecord\\|ObjectiveFunction\\|"
        "Obstruction\\|OctalStringMonoid\\|"
        "OddGraph\\|OldSubspace\\|"
        "Omega\\|OmegaMinus\\|"
        "OmegaPlus\\|One\\|"
        "Open\\|OpenGraphFile\\|"
        "OpenSmallGroupDatabase\\|Operands\\|"
        "Operator\\|OppositeAlgebra\\|"
        "OptimalSkewness\\|OptimizedRepresentation\\|"
        "Orbit\\|OrbitAction\\|"
        "OrbitActionBounded\\|OrbitBounded\\|"
        "OrbitClosure\\|OrbitImage\\|"
        "OrbitImageBounded\\|OrbitKernel\\|"
        "OrbitKernelBounded\\|OrbitalGraph\\|"
        "Orbits\\|OrbitsOfSpaces\\|"
        "OrbitsPartition\\|Order\\|"
        "OrderDRK\\|OrderFromHandle\\|"
        "OrderPR\\|OrderWithGram\\|"
        "OrderedMonoid\\|Ordering\\|"
        "Orders\\|OrientatedGraph\\|"
        "Origin\\|OriginalRing\\|"
        "OrthogonalSum\\|Orthogonalize\\|"
        "OrthogonalizeGram\\|Orthonormalize\\|"
        "OutDegree\\|OutEdges\\|"
        "OutNeighbors\\|OutNeighbours\\|"
        "OuterFPGroup\\|OuterFaces\\|"
        "OuterOrder\\|OuterShape\\|"
        "OuterVertices\\|OvalDerivation\\|"
        "OverDimension\\|OverRing\\|"
        "Overworld\\|P1Action\\|"
        "P1Classes\\|P1Reduce\\|"
        "PGL\\|POpen\\|"
        "PSL\\|PSL2\\|"
        "PackageConflicts\\|PackageList\\|"
        "PackageStatus\\|PairReduce\\|"
        "PairReduceGram\\|PaleyGraph\\|"
        "PaleyTournament\\|ParallelClass\\|"
        "ParallelClasses\\|ParallelSort\\|"
        "Parameters\\|Parametrization\\|"
        "ParametrizationMatrix\\|Parent\\|"
        "ParentGraph\\|ParentPlane\\|"
        "ParentRing\\|Pari\\|"
        "ParityCheckMatrix\\|Partition\\|"
        "PartitionCovers\\|Partitions\\|"
        "PascalTriangle\\|PatchGerms\\|"
        "PathGraph\\|PathTree\\|"
        "Pencil\\|PerfectGroupDatabase\\|"
        "PerfectSubgroups\\|PeriodMapping\\|"
        "Periods\\|PermRepLimit\\|"
        "Permutation\\|PermutationCode\\|"
        "PermutationGroup\\|PermutationMatrix\\|"
        "PermutationModule\\|PermutationRepresentation\\|"
        "Permutations\\|PermuteSequence\\|"
        "PhiLog\\|PhiOmega\\|"
        "Pi\\|PicardGroup\\|"
        "PicardNumber\\|Pipe\\|"
        "Place\\|Places\\|"
        "PlacticMonoid\\|PlotkinBound\\|"
        "PlotkinSum\\|Point\\|"
        "PointDegree\\|PointDegrees\\|"
        "PointEnumeration\\|PointGraph\\|"
        "PointGroup\\|PointSet\\|"
        "Points\\|PointsAtInfinity\\|"
        "PointsFiniteField\\|PointsKnown\\|"
        "PolarToComplex\\|Poles\\|"
        "PollardRho\\|PolyMapKernel\\|"
        "PolygonGraph\\|Polylog\\|"
        "Polynomial\\|PolynomialAlgebra\\|"
        "PolynomialMap\\|PolynomialPair\\|"
        "PolynomialRing\\|Polynomials\\|"
        "Position\\|PositiveConjugates\\|"
        "PositiveSum\\|Power\\|"
        "PowerFormalSet\\|PowerGroup\\|"
        "PowerIdeal\\|PowerIndexedSet\\|"
        "PowerMap\\|PowerMultiset\\|"
        "PowerProduct\\|PowerRelation\\|"
        "PowerResidueCode\\|PowerSequence\\|"
        "PowerSeries\\|PowerSeriesAlgebra\\|"
        "PowerSeriesRing\\|PowerSet\\|"
        "PowerStructure\\|PrePatchMaps\\|"
        "Precision\\|PrecisionBound\\|"
        "Predec\\|PreimageIdeal\\|"
        "PreimageRing\\|PreparataCode\\|"
        "Preprune\\|Presentation\\|"
        "PresentationLength\\|PrimalityCertificate\\|"
        "Primary\\|PrimaryAlgebra\\|"
        "PrimaryBasis\\|PrimaryComponents\\|"
        "PrimaryDecomposition\\|PrimaryIdeal\\|"
        "PrimaryInvariants\\|Prime\\|"
        "PrimeBasis\\|PrimeComponents\\|"
        "PrimeDecomposition\\|PrimeDivisors\\|"
        "PrimeField\\|PrimeForm\\|"
        "PrimeIdeal\\|PrimePolynomials\\|"
        "PrimeRing\\|Primes\\|"
        "PrimesUpTo\\|PrimitiveElement\\|"
        "PrimitiveGroup\\|PrimitiveGroups\\|"
        "PrimitivePart\\|PrimitiveQuotient\\|"
        "PrimitiveRoot\\|PrincipalCharacter\\|"
        "PrincipalDivisor\\|PrincipalIdealMap\\|"
        "PrintBase\\|PrintCoding\\|"
        "PrintCollector\\|PrintCompact\\|"
        "PrintExtensions\\|PrintFile\\|"
        "PrintFileMagma\\|PrintGenerators\\|"
        "PrintLength\\|PrintMapping\\|"
        "PrintModules\\|PrintPairs\\|"
        "PrintPlace\\|PrintPrimes\\|"
        "PrintProcess\\|PrintQuotient\\|"
        "PrintRelat\\|PrintRelators\\|"
        "PrintSeries\\|PrintSize\\|"
        "PrintStatus\\|PrintTermsOfDegree\\|"
        "PrintToPrecision\\|PrintVSrfK3\\|"
        "Process\\|Product\\|"
        "Proj\\|Projection\\|"
        "ProjectionChains\\|ProjectiveClosure\\|"
        "ProjectiveCover\\|ProjectiveCurve\\|"
        "ProjectiveMap\\|ProjectiveModule\\|"
        "ProjectiveOmega\\|ProjectiveOrder\\|"
        "ProjectivePatchMap\\|ProjectivePlane\\|"
        "ProjectiveResolution\\|ProjectiveSpace\\|"
        "Projectivity\\|Prune\\|"
        "PseudoAdd\\|PseudoAddMultiple\\|"
        "PseudoBasis\\|PseudoDimension\\|"
        "PseudoGcd\\|PseudoGenerators\\|"
        "PseudoInverse\\|PseudoQuotrem\\|"
        "PseudoRemainder\\|PseudoXGcd\\|"
        "Psi\\|PuiseuxExpansion\\|"
        "PuiseuxExponents\\|PuiseuxSeriesRing\\|"
        "Pullback\\|PunctureCode\\|"
        "PuncturedCode\\|PureLattice\\|"
        "PushThroughIsogeny\\|Pushout\\|"
        "Put\\|PutI\\|"
        "PutICount\\|Puts\\|"
        "QIdeal\\|QMatrix\\|"
        "QRCode\\|QSpace\\|"
        "QuadraticField\\|QuadraticForm\\|"
        "QuadraticForms\\|QuadraticModule\\|"
        "QuadraticOrder\\|QuadraticSpace\\|"
        "QuadraticTwist\\|QuadraticTwists\\|"
        "QuasiCyclicCode\\|QuaternionAlgebra\\|"
        "QuaternionAlgebraDatabase\\|QuaternionOrder\\|"
        "QuaternionicMatrixGroupDatabase\\|Quotient\\|"
        "QuotientComplex\\|QuotientMap\\|"
        "QuotientMapMatrix\\|QuotientModule\\|"
        "QuotientRelations\\|QuotientRing\\|"
        "QuotientSubgroup\\|Quotrem\\|"
        "RCLazySeries\\|RHS\\|"
        "RMatrixSpace\\|RModule\\|"
        "RModuleWithBasis\\|RSACryptosystem\\|"
        "RSAExponent\\|RSAModulus\\|"
        "RSKCorrespondence\\|RSpace\\|"
        "RSpaceWithBasis\\|RSpaceWithModuli\\|"
        "RTCS\\|RWS\\|"
        "RWSGroup\\|RWSMonoid\\|"
        "Radical\\|RadicalExtension\\|"
        "RadicalQuotient\\|RamificationDegree\\|"
        "RamificationField\\|RamificationGroup\\|"
        "RamificationIndex\\|RamifiedPrimes\\|"
        "Random\\|RandomBaseChange\\|"
        "RandomCFP\\|RandomDigraph\\|"
        "RandomFib\\|RandomGraph\\|"
        "RandomHookWalk\\|RandomKey\\|"
        "RandomKeys\\|RandomLinearCode\\|"
        "RandomPartition\\|RandomPlace\\|"
        "RandomProcess\\|RandomSchreier\\|"
        "RandomSequenceRSA\\|RandomSubcomplex\\|"
        "RandomTableau\\|RandomTree\\|"
        "RandomWord\\|Rank\\|"
        "RankBounds\\|RankPoints\\|"
        "RationalCurve\\|RationalCusps\\|"
        "RationalField\\|RationalForm\\|"
        "RationalFunction\\|RationalFunctions\\|"
        "RationalMap\\|RationalMapping\\|"
        "RationalMaps\\|RationalMatrixGroupDatabase\\|"
        "RationalPoint\\|RationalPointSieve\\|"
        "RationalPoints\\|Rationals\\|"
        "Ratpoints\\|RawK3Surface\\|"
        "RayClassField\\|RayClassGroup\\|"
        "RayResidueRing\\|Re\\|"
        "Reachable\\|Read\\|"
        "ReadBytes\\|Real\\|"
        "RealField\\|RealPeriod\\|"
        "RealRoots\\|RealTamagawaNumber\\|"
        "RealVolume\\|Realtime\\|"
        "RecognizeClassical\\|Rectify\\|"
        "RecursiveGrphRes\\|RedoEnumeration\\|"
        "Reduce\\|ReduceCharacters\\|"
        "ReduceGenerators\\|ReduceMatrix\\|"
        "ReduceVector\\|ReducedBasis\\|"
        "ReducedDegree\\|ReducedForms\\|"
        "ReducedGramMatrix\\|ReducedLegendreModel\\|"
        "ReducedLegendrePolynomial\\|ReducedModel\\|"
        "ReducedNorm\\|ReducedOrbits\\|"
        "ReducedPoint\\|ReducedResultant\\|"
        "ReducedSubscheme\\|ReducedTrace\\|"
        "Reduction\\|ReductionOrbit\\|"
        "ReductionStep\\|ReductionType\\|"
        "Reductions\\|Reductum\\|"
        "ReedMullerCode\\|ReedSolomonCode\\|"
        "RefineSection\\|ReflectionGroup\\|"
        "Regexp\\|RegularSubgroups\\|"
        "Regulator\\|ReidNumber\\|"
        "RelationIdeal\\|RelationMatrix\\|"
        "Relations\\|RelativeField\\|"
        "RelativePrecision\\|RelevantPrimes\\|"
        "Remove\\|RemoveCentres\\|"
        "RemoveConstraint\\|RemoveEdge\\|"
        "RemoveEdges\\|RemoveFactor\\|"
        "RemoveFiles\\|RemoveHypersurface\\|"
        "RemoveIrreducibles\\|RemoveVertex\\|"
        "RemoveVertices\\|RemoveWeight\\|"
        "Rep\\|RepetitionCode\\|"
        "ReplacePrimes\\|ReplaceRelation\\|"
        "ReplicationNumber\\|RepnProcessExtract\\|"
        "RepnProcessNReps\\|Representation\\|"
        "RepresentationSum\\|RepresentationType\\|"
        "Representations\\|Representative\\|"
        "Representatives\\|Res_H2_G_QmodZ\\|"
        "Residual\\|ResidualRoots\\|"
        "Residue\\|ResidueClassDegree\\|"
        "ResidueClassField\\|ResidueClassRing\\|"
        "ResidueCode\\|ResidueField\\|"
        "ReslistField\\|Resolution\\|"
        "ResolutionGraph\\|ResolutionSpine\\|"
        "ResolvedDualFan\\|Resolvent\\|"
        "Restrict\\|RestrictField\\|"
        "Restriction\\|RestrictionToPatch\\|"
        "Resultant\\|ResumeEnumeration\\|"
        "Retrieve\\|Retype\\|"
        "Reverse\\|ReverseFilling\\|"
        "Reversion\\|RevertClass\\|"
        "Rewind\\|Rewrite\\|"
        "RewriteSystem\\|ReynoldsOperator\\|"
        "RiemannRochBasis\\|RiemannRochSpace\\|"
        "RightAction\\|RightAnnihilator\\|"
        "RightCosetSpace\\|RightGCD\\|"
        "RightGcd\\|RightIdeal\\|"
        "RightIdealClasses\\|RightIdeals\\|"
        "RightIsomorphism\\|RightLCM\\|"
        "RightLcm\\|RightNormalForm\\|"
        "RightOrder\\|RightRegularModule\\|"
        "RightRing\\|RightTransversal\\|"
        "RightZeroExtension\\|Ring\\|"
        "RingMap\\|RingOfIntegers\\|"
        "RingOfModularForms\\|RombergQuadrature\\|"
        "Root\\|RootNormalization\\|"
        "RootOfUnity\\|RootSide\\|"
        "RootSystemMatrix\\|RootVertex\\|"
        "Roots\\|RootsHack\\|"
        "RootsNonExact\\|Rotate\\|"
        "RotateRows\\|RotateWord\\|"
        "Round\\|RoundFour\\|"
        "RoundReal\\|Row\\|"
        "RowInsert\\|RowLength\\|"
        "RowList\\|RowNullSpace\\|"
        "RowSkewLength\\|RowSpace\\|"
        "RowSubmatrix\\|RowSubmatrixRange\\|"
        "RowWeight\\|RowWeights\\|"
        "RowWord\\|Rows\\|"
        "Rowspace\\|RuledSurface\\|"
        "SEA\\|SHA1\\|"
        "SIntegralPoints\\|SUnitGroup\\|"
        "SatohTrace\\|Saturation\\|"
        "ScalarField\\|ScalarMatrix\\|"
        "ScalarPowerSeries\\|ScalarsUnitaryForm\\|"
        "ScaledGenus\\|ScaledLattice\\|"
        "ScalingForm\\|Scheme\\|"
        "SchemeHomogenize\\|SchemeMorphism\\|"
        "SchemeThrough\\|SchreierGenerators\\|"
        "SchreierGraph\\|SchreierSystem\\|"
        "SchreierVector\\|SchreierVectors\\|"
        "Schreyer\\|Schur\\|"
        "Search\\|SearchEqual\\|"
        "Sec\\|Sech\\|"
        "SecondaryModule\\|SectionCentraliser\\|"
        "SectionCentralizer\\|Sections\\|"
        "Seek\\|SelectAlgo\\|"
        "Self\\|Selfintersection\\|"
        "Selfintersections\\|SemiLinearGroup\\|"
        "Semidir\\|SeparatingElement\\|"
        "SeparationVertices\\|SequenceToElement\\|"
        "SequenceToList\\|SequenceToMultiset\\|"
        "SequenceToSet\\|SeriesHenselLift\\|"
        "SeriesRoot\\|SeriesRoots\\|"
        "SerreBound\\|Set\\|"
        "SetAFR\\|SetArrows\\|"
        "SetBaseGerm\\|SetBufferSize\\|"
        "SetCanonicalClass\\|SetDefining\\|"
        "SetDisplayLevel\\|SetEntry\\|"
        "SetGroupLaw\\|SetHeckeBound\\|"
        "SetIdentifiers\\|SetKantPrecision\\|"
        "SetKantVerbose\\|SetLibraries\\|"
        "SetLibraryRoot\\|SetLogFile\\|"
        "SetLowerBound\\|SetMultiplicities\\|"
        "SetOptions\\|SetOrderMaximal\\|"
        "SetOutputFile\\|SetPath\\|"
        "SetPointPoint\\|SetPowerPrinting\\|"
        "SetPrePatchMaps\\|SetPrecision\\|"
        "SetPresentation\\|SetPrintLevel\\|"
        "SetPrinting\\|SetPrompt\\|"
        "SetToIndexedSet\\|SetToMultiset\\|"
        "SetToSequence\\|SetTrace\\|"
        "SetUpperBound\\|SetVerbose\\|"
        "Setseq\\|Seysen\\|"
        "SeysenGram\\|SgpFPElt\\|"
        "Shadow\\|ShadowSpace\\|"
        "Shape\\|ShephardTodd\\|"
        "Shift\\|ShiftToDegreeZero\\|"
        "ShiftVal\\|ShiftValuation\\|"
        "ShimuraGenus\\|ShimuraSubgroup\\|"
        "ShortBasis\\|ShortVectors\\|"
        "ShortVectorsMatrix\\|ShortenCode\\|"
        "ShortestVectors\\|Shoup\\|"
        "ShowOptions\\|ShowRelshp\\|"
        "ShrikhandeGraph\\|ShrinkingGenerator\\|"
        "ShrinkingGeneratorCryptosystem\\|Sieve\\|"
        "Sign\\|SignDecomposition\\|"
        "Signature\\|SignatureList\\|"
        "Signatures\\|SilvermanBound\\|"
        "SilvermanBounds\\|SimpleExtension\\|"
        "SimpleGroupName\\|SimpleGroupOrder\\|"
        "SimpleModule\\|SimpleProjection\\|"
        "SimpleSubgroups\\|Simplex\\|"
        "SimplexCode\\|SimplifiedModel\\|"
        "Simplify\\|SimplifyLength\\|"
        "SimpsonQuadrature\\|SimsSchreier\\|"
        "SimsSchreierCoding\\|Sin\\|"
        "Sincos\\|Single\\|"
        "SingletonBound\\|Singular\\|"
        "SingularFibres\\|SingularPoints\\|"
        "SingularSubscheme\\|Sinh\\|"
        "Size\\|Skew\\|"
        "SkewShape\\|SkewWeight\\|"
        "Slope\\|Slopes\\|"
        "SmallGroup\\|SmallGroups\\|"
        "SmithForm\\|Smod\\|"
        "Socket\\|Socle\\|"
        "SocleAction\\|SocleFactor\\|"
        "SocleFactors\\|SocleImage\\|"
        "SocleKernel\\|SocleQuotient\\|"
        "SocleSeries\\|SolRepProc\\|"
        "SolubleSubgroups\\|Solution\\|"
        "SolutionSpace\\|Solutions\\|"
        "SolvableSubgroups\\|Sort\\|"
        "SortDecomposition\\|Span\\|"
        "SpanningBasis\\|SpanningForest\\|"
        "SpanningTree\\|SparseMatrix\\|"
        "Spec\\|SpecialLinearGroup\\|"
        "SpecialWeights\\|Specialization\\|"
        "Spectrum\\|Sphere\\|"
        "SpherePackingBound\\|SphereVolume\\|"
        "SphericalMatrix\\|Spin\\|"
        "SpinAction\\|SpinorCharacters\\|"
        "SpinorGenera\\|SpinorGenerators\\|"
        "SpinorGenus\\|Spiral\\|"
        "Splice\\|SpliceDiagram\\|"
        "Split\\|SplitCollector\\|"
        "SplitExtension\\|SplitPrimes\\|"
        "SplitSection\\|Splitcomponents\\|"
        "SplittingField\\|Sprint\\|"
        "Sqrt\\|SquareFree\\|"
        "SquareFreeFactorization\\|SquareLatticeGraph\\|"
        "SquareRelations\\|SquareRoot\\|"
        "SrfKumPt\\|SrivastavaCode\\|"
        "Stabilizer\\|StandardCusp\\|"
        "StandardForm\\|StandardFormField\\|"
        "StandardFormInfo\\|StandardGraph\\|"
        "StandardGroup\\|StandardLattice\\|"
        "StandardTableaux\\|StarInvolution\\|"
        "States\\|SteenrodOperation\\|"
        "SteinitzClass\\|SteinitzForm\\|"
        "Steps\\|Strassen\\|"
        "String\\|StringMonoid\\|"
        "StringToCode\\|StringToInteger\\|"
        "Strings\\|Strip\\|"
        "StripEncoding\\|StrippedCoding\\|"
        "StrongGenerators\\|StronglyRegularGraphsDatabase\\|"
        "StructureConstant\\|StructureConstants\\|"
        "SubModule\\|SubOrder\\|"
        "Subcode\\|SubcodeBetweenCode\\|"
        "Subcomplex\\|SubfieldCode\\|"
        "SubfieldLattice\\|SubfieldSubcode\\|"
        "SubfieldSubplane\\|Subfields\\|"
        "Subgraph\\|Subgroup\\|"
        "SubgroupClasses\\|SubgroupLattice\\|"
        "SubgroupOfTorus\\|SubgroupScheme\\|"
        "SubgroupToMatrix\\|Subgroups\\|"
        "Sublattices\\|Submatrix\\|"
        "SubmatrixRange\\|Submodule\\|"
        "SubmoduleAction\\|SubmoduleImage\\|"
        "SubmoduleLattice\\|SubmoduleWithBasis\\|"
        "Submodules\\|SubnormalSeries\\|"
        "Subring\\|Subsequences\\|"
        "Subsets\\|SubspaceStabilizer\\|"
        "Substitute\\|SubstitutionCryptosystem\\|"
        "SubstitutionEnciphering\\|Substring\\|"
        "SubstructureMaps\\|Subword\\|"
        "SuccessiveMinima\\|SuggestedPrecision\\|"
        "Sum\\|SumNorm\\|"
        "SumOfDivisors\\|Summation\\|"
        "SuperScheme\\|SuperSummitInfimum\\|"
        "SuperSummitProcess\\|SuperSummitSet\\|"
        "SupersingularBasis\\|SupersingularEllipticCurve\\|"
        "SupersingularInvariants\\|SupersingularPolynomial\\|"
        "SupersingularPolynomials\\|Supplement\\|"
        "Supplements\\|Support\\|"
        "SupportedSubcode\\|Supremum\\|"
        "SuzukiGroup\\|SwapColumns\\|"
        "SwapRows\\|SwinnertonDyerPolynomial\\|"
        "Switch\\|Sylow\\|"
        "SylowBasis\\|SylowSubgroup\\|"
        "SylvesterMatrix\\|Sym\\|"
        "SymElt\\|SymmetricCharacter\\|"
        "SymmetricForms\\|SymmetricGroup\\|"
        "SymmetricMatrix\\|SymmetricSquare\\|"
        "Symmetrization\\|SymplecticForm\\|"
        "SymplecticGroup\\|System\\|"
        "SystemNormaliser\\|SystemNormalizer\\|"
        "SyzygyMatrix\\|SyzygyModule\\|"
        "TabPrint\\|TamagawaNumber\\|"
        "TamagawaNumbers\\|Tan\\|"
        "Tangent\\|TangentAngle\\|"
        "TangentCone\\|TangentLine\\|"
        "TangentSpace\\|Tanh\\|"
        "TeichmuellerSystem\\|TensorBasis\\|"
        "TensorFactors\\|TensorInducedBasis\\|"
        "TensorPower\\|TensorProduct\\|"
        "TernaryModules\\|TernaryTheta\\|"
        "ThetaOperator\\|ThetaSeries\\|"
        "ThreeSelmerGroup\\|Thue\\|"
        "ThueEval\\|ThueObject\\|"
        "ThueSolve\\|TietzeProcess\\|"
        "TitsGroup\\|ToddCoxeter\\|"
        "Top\\|TopQuotients\\|"
        "TorsionBasis\\|TorsionBound\\|"
        "TorsionFreeRank\\|TorsionFreeSubmodule\\|"
        "TorsionInvariants\\|TorsionSubgroup\\|"
        "TorsionSubgroupScheme\\|TorsionSubmodule\\|"
        "TorsionUnitGroup\\|TotalDegree\\|"
        "TotalLinking\\|TotallyRamifiedExtension\\|"
        "Trace\\|TraceAbs\\|"
        "TraceMatrix\\|TraceOfFrobenius\\|"
        "TracesOfFrobenius\\|TrailingTerm\\|"
        "Transformation\\|TransitiveGroup\\|"
        "TransitiveGroups\\|TransitiveQuotient\\|"
        "Transitivity\\|Translation\\|"
        "TranslationMap\\|TranslationMatches\\|"
        "Transpose\\|TranspositionCryptosystem\\|"
        "Transversal\\|TransversalProcess\\|"
        "TrialDivision\\|TriangularGraph\\|"
        "TrivialModule\\|Truncation\\|"
        "Tschirnhausen\\|Tup\\|"
        "TupleToList\\|Tuplist\\|"
        "TwistedQRCode\\|TwistedThetaSeries\\|"
        "Twists\\|TwoElement\\|"
        "TwoElementNormal\\|TwoGenerators\\|"
        "TwoSelmerGroupData\\|TwoSidedIdealClasses\\|"
        "TwoTorsionPolynomial\\|TwoTorsionSubgroup\\|"
        "Type\\|Types\\|"
        "Undefine\\|UniformizingParameter\\|"
        "Union\\|UniqueRoot\\|"
        "UnitEquation\\|UnitGenerators\\|"
        "UnitGroup\\|UnitRank\\|"
        "UnitSquareClass\\|UnitaryForm\\|"
        "Units\\|Unity\\|"
        "UnivariatePolynomial\\|UniversalMap\\|"
        "Universe\\|UniverseCode\\|"
        "UnprojectionChains\\|Unprojections\\|"
        "UnramifiedCover\\|UnsetBounds\\|"
        "UpdateLevels\\|UpperCentralSeries\\|"
        "UpperHalfPlane\\|UseFlag\\|"
        "UserBasePoints\\|UserGenerators\\|"
        "UserMapCreateRaw\\|UserRepresentation\\|"
        "UsesBrandt\\|Valence\\|"
        "Valency\\|Valuation\\|"
        "ValuationRing\\|ValuationStar\\|"
        "ValuationsOfRoots\\|ValueList\\|"
        "ValueMap\\|ValueRing\\|"
        "VanLintBound\\|VariableExtension\\|"
        "VariableWeights\\|Variety\\|"
        "VarietySequence\\|Vector\\|"
        "VectorSpace\\|VeluImageCurve\\|"
        "VeluIsogeny\\|Verify\\|"
        "Verschiebung\\|Vertex\\|"
        "VertexConnectivity\\|VertexLabel\\|"
        "VertexLabels\\|VertexPath\\|"
        "VertexSeparator\\|VertexSet\\|"
        "VerticalJoin\\|Vertices\\|"
        "VigenereCryptosystem\\|VoidLocus\\|"
        "Volume\\|VoronoiCell\\|"
        "VoronoiGraph\\|WanOperator\\|"
        "WeakApproximation\\|WeberClassPolynomial\\|"
        "WeberF\\|WeberF2\\|"
        "WeberFn\\|WeberPolynomial\\|"
        "WeierstrassModel\\|WeierstrassPlaces\\|"
        "WeierstrassPoints\\|WeierstrassPreparation\\|"
        "WeierstrassSeries\\|WeierstrassTate\\|"
        "Weight\\|WeightClass\\|"
        "WeightDistribution\\|WeightEnumerator\\|"
        "WeightedDegree\\|Weights\\|"
        "WeilHeight\\|WeilPairing\\|"
        "WeilRestriction\\|Widths\\|"
        "WindingElement\\|WindingLattice\\|"
        "WindingSubmodule\\|WittDesign\\|"
        "WittInvariant\\|WittRing\\|"
        "Word\\|WordGroup\\|"
        "WordStrip\\|WordToSequence\\|"
        "Words\\|WreathProduct\\|"
        "Write\\|WronskianOrders\\|"
        "XGCD\\|Xgcd\\|"
        "Zero\\|ZeroCode\\|"
        "ZeroMap\\|ZeroModule\\|"
        "ZeroSubspace\\|Zeros\\|"
        "ZetaFunction\\|ZinovievCode\\|"
        "aInvariants\\|bInvariants\\|"
        "cInvariants\\|jInvariant\\|"
        "jInvariantDatabase\\|jInvariantDatabase\\|"
        "pAdicDiagonalization\\|pAdicEllipticLogarithm\\|"
        "pAdicEmbeddings\\|pAdicField\\|"
        "pAdicRing\\|pCentralSeries\\|"
        "pClass\\|pCore\\|"
        "pCover\\|pCoveringGroup\\|"
        "pFundamentalUnits\\|pGenericSubgroup\\|"
        "pIntegralModel\\|pInverse\\|"
        "pMaximalOrder\\|pMaximalSuperLattice\\|"
        "pMinimalWeierstrassModel\\|pMultiplicator\\|"
        "pMultiplicatorRank\\|pNewModularDegree\\|"
        "pNormalModel\\|pPrimaryComponent\\|"
        "pPrimaryInvariants\\|pQuotient\\|"
        "pQuotientProcess\\|pRadical\\|"
        "pRank\\|pRanks\\|"
        "pSelmerGroup\\|pSubgroup\\|"
        "qEigenform\\|qExpansion\\|"
        "qExpansionBasis\\|qIntegralBasis\\|"
        "\\)\\>")
      'font-lock-function-name-face)

      ;; Constructors.
      (cons (concat "\\<\\("
        "ideal\\|hom\\|map\\|quo\\|rec\\|recformat\\|sub\\|"
        "\\)\\>")
      'font-lock-builtin-face)

      ;; Category names
      (cons (concat "\\<\\("
        "Aff\\|Alg\\|AlgAss\\|AlgAssElt\\|AlgBas\\|"
        "AlgBasElt\\|AlgChtr\\|AlgChtrElt\\|AlgClff\\|AlgClffElt\\|"
        "AlgElt\\|AlgExt\\|AlgExtElt\\|AlgFP\\|AlgFPElt\\|"
        "AlgFPG\\|AlgFPGElt\\|AlgFPRes\\|AlgFPResElt\\|AlgGen\\|"
        "AlgGenElt\\|AlgGrp\\|AlgGrpElt\\|AlgGrpSub\\|AlgHck\\|"
        "AlgHckElt\\|AlgLie\\|AlgLieElt\\|AlgMat\\|AlgMatElt\\|"
        "AlgQuat\\|AlgQuatElt\\|AlgQuatOrd\\|AlgQuatOrdElt\\|AlgSym\\|"
        "AlgSymElt\\|Aut\\|AutCrv\\|AutCrvEll\\|AutCrvGrp\\|"
        "Bool\\|BoolElt\\|Cat\\|Clstr\\|Code\\|"
        "Cop\\|CopElt\\|CosetGeom\\|Crv\\|CrvCon\\|"
        "CrvEll\\|CrvGrp\\|CrvHyp\\|CrvMod\\|CrvRat\\|"
        "Crypt\\|CryptKey\\|CryptTxt\\|Cyc\\|DB\\|"
        "DBUser\\|DiffFun\\|DiffFunElt\\|DivCrv\\|DivCrvElt\\|"
        "DivFun\\|DivFunElt\\|DivNum\\|DivNumElt\\|DivSch\\|"
        "DivSchElt\\|Fbr\\|Fld\\|FldAC\\|FldACElt\\|"
        "FldAb\\|FldAbFun\\|FldAlg\\|FldAlgElt\\|FldCom\\|"
        "FldComElt\\|FldCyc\\|FldCycElt\\|FldElt\\|FldFin\\|"
        "FldFinElt\\|FldFun\\|FldFunElt\\|FldFunG\\|FldFunGElt\\|"
        "FldFunRat\\|FldFunRatElt\\|FldFunRatMElt\\|FldFunRatUElt\\|FldLoc\\|"
        "FldLocElt\\|FldNum\\|FldNumElt\\|FldOrd\\|FldOrdElt\\|"
        "FldPad\\|FldPadElt\\|FldPr\\|FldPrElt\\|FldQuad\\|"
        "FldQuadElt\\|FldRat\\|FldRatElt\\|FldRe\\|FldReElt\\|"
        "FldResLst\\|FldResLstElt\\|GSet\\|GSetEnum\\|Grp\\|"
        "GrpAb\\|GrpAbElt\\|GrpAbGen\\|GrpAbGenElt\\|GrpAtc\\|"
        "GrpAtcElt\\|GrpAut\\|GrpAutElt\\|GrpAuto\\|GrpAutoElt\\|"
        "GrpBrd\\|GrpBrdElt\\|GrpCayg\\|GrpCaygElt\\|GrpDrch\\|"
        "GrpDrchElt\\|GrpElt\\|GrpFP\\|GrpFPCos\\|GrpFPCosElt\\|"
        "GrpFPCox\\|GrpFPCoxElt\\|GrpFPDcos\\|GrpFPDcosElt\\|GrpFPElt\\|"
        "GrpFrml\\|GrpFrmlElt\\|GrpGPC\\|GrpGPCElt\\|GrpLie\\|"
        "GrpLieElt\\|GrpMat\\|GrpMatElt\\|GrpMatProj\\|GrpMatProjElt\\|"
        "GrpPC\\|GrpPCElt\\|GrpPSL2\\|GrpPSL2Elt\\|GrpPerm\\|"
        "GrpPermCox\\|GrpPermElt\\|GrpSLP\\|GrpSLPElt\\|Grph\\|"
        "GrphDir\\|GrphEdge\\|GrphEdgeSet\\|GrphMultDir\\|GrphMultUnd\\|"
        "GrphUnd\\|GrphVert\\|GrphVertSet\\|Hackobj\\|HatHat\\|"
        "HomCrv\\|HomCrvEll\\|HomCrvGrp\\|HomGrp\\|Infty\\|"
        "IsoCrv\\|IsoCrvEll\\|IsoCrvGrp\\|Jac\\|JacHyp\\|"
        "JacHypPt\\|Lat\\|LatElt\\|List\\|MPI\\|"
        "Map\\|MapAutSch\\|MapChn\\|MapCrv\\|MapCrvEll\\|"
        "MapCrvEll\\|MapCrvGrp\\|MapIsoCrv\\|MapIsoCrvEll\\|MapIsoCrvGrp\\|"
        "MapIsoSch\\|MapSch\\|Mod\\|ModAlg\\|ModAlgElt\\|"
        "ModAlt\\|ModAltElt\\|ModBrdt\\|ModBrdtElt\\|ModCoho\\|"
        "ModCpx\\|ModCyc\\|ModCycElt\\|ModDed\\|ModDedElt\\|"
        "ModED\\|ModEDElt\\|ModElt\\|ModExt\\|ModExtElt\\|"
        "ModFld\\|ModFldElt\\|ModFrm\\|ModFrmElt\\|ModGrp\\|"
        "ModGrpElt\\|ModHgn\\|ModHgnElt\\|ModHrm\\|ModHrmElt\\|"
        "ModLat\\|ModLatElt\\|ModMPol\\|ModMPolElt\\|ModMatFld\\|"
        "ModMatFldElt\\|ModMatGrp\\|ModMatGrpElt\\|ModMatRng\\|ModMatRngElt\\|"
        "ModRng\\|ModRngElt\\|ModSS\\|ModSSElt\\|ModShf\\|"
        "ModShfElt\\|ModSym\\|ModSymElt\\|ModTheta\\|ModThetaElt\\|"
        "ModTupFld\\|ModTupFldElt\\|ModTupRng\\|ModTupRngElt\\|MonAb\\|"
        "MonAbElt\\|MonAtc\\|MonAtcElt\\|MonDivLst\\|MonFP\\|"
        "MonFPElt\\|MonOrd\\|MonOrdElt\\|MonPlc\\|MonPlcElt\\|"
        "MonStg\\|MonStgElt\\|MonStgGen\\|MonStgGenElt\\|NwtnPgon\\|"
        "NwtnPgonFace\\|PlcCrv\\|PlcCrvElt\\|PlcEnum\\|PlcFun\\|"
        "PlcFunElt\\|PlcNum\\|PlcNumElt\\|PlcRat\\|PlcRatElt\\|"
        "PowAut\\|PowAutSch\\|PowGSetEnum\\|PowGrpAlgSub\\|PowIdl\\|"
        "PowIsoSch\\|PowMap\\|PowMapAut\\|PowMapSch\\|PowModTupRng\\|"
        "PowSeqEnum\\|PowSetEnum\\|PowSetFormal\\|PowSetIndx\\|PowSetMulti\\|"
        "PowStr\\|PowerGroup\\|Prj\\|PrjProd\\|PrjScrl\\|"
        "Pt\\|PtEll\\|PtGrp\\|PtHyp\\|PtJac\\|"
        "PtSrfEll\\|QuadBin\\|QuadBinElt\\|Rec\\|Rng\\|"
        "RngArt\\|RngArtElt\\|RngArtIdl\\|RngCyc\\|RngCycElt\\|"
        "RngDiffOp\\|RngDiffOpElt\\|RngDiffOpLIdl\\|RngDiffOpRIdl\\|RngElt\\|"
        "RngFrm\\|RngFrmElt\\|RngFunG\\|RngFunGElt\\|RngFunOrd\\|"
        "RngFunOrdElt\\|RngFunOrdG\\|RngFunOrdGElt\\|RngFunOrdIdl\\|RngGal\\|"
        "RngGalElt\\|RngHck\\|RngHckElt\\|RngInt\\|RngIntElt\\|"
        "RngIntEltFact\\|RngIntRes\\|RngIntResElt\\|RngInvar\\|RngLoc\\|"
        "RngLocElt\\|RngMPol\\|RngMPolElt\\|RngMPolLoc\\|RngMPolRes\\|"
        "RngMPolResElt\\|RngMPolSub\\|RngOrd\\|RngOrdElt\\|RngOrdFracIdl\\|"
        "RngOrdIdl\\|RngOrdRes\\|RngOrdResElt\\|RngPad\\|RngPadElt\\|"
        "RngPadRes\\|RngPadResElt\\|RngPadResExt\\|RngPadResExtElt\\|RngPowLaz\\|"
        "RngPowLazElt\\|RngQuad\\|RngQuadElt\\|RngQuadFracIdl\\|RngQuadIdl\\|"
        "RngSer\\|RngSerElt\\|RngSerLaur\\|RngSerLaurElt\\|RngSerPow\\|"
        "RngSerPowElt\\|RngSerPuis\\|RngSerPuisElt\\|RngUPol\\|RngUPolElt\\|"
        "RngUPolRes\\|RngUPolResElt\\|RngVal\\|RngValElt\\|RngWitt\\|"
        "RngWittElt\\|RootDtm\\|RootSys\\|Sch\\|SchGrp\\|"
        "SchGrpEll\\|SeqEnum\\|Set\\|SetCart\\|SetCsp\\|"
        "SetCspElt\\|SetEnum\\|SetFormal\\|SetGIndx\\|SetIndx\\|"
        "SetMulti\\|SetPt\\|SetPtEll\\|SetPtGrp\\|SetPtHyp\\|"
        "SetPtJac\\|SetPtSrfEll\\|Setq\\|Shf\\|Socket\\|"
        "SpcFld\\|SpcFldElt\\|SpcHyp\\|SpcHypArc\\|SpcHypElt\\|"
        "SpcHypPgon\\|SpcRng\\|SpcRngElt\\|SrfEll\\|SrfKum\\|"
        "SrfKumPt\\|Srfc\\|Str\\|SubFldLat\\|SubFldLatElt\\|"
        "SubGrpLat\\|SubGrpLatElt\\|SubModLat\\|SubModLatElt\\|SymAlgQuat\\|"
        "SymCrvEll\\|SymFry\\|SymGen\\|SymGenLoc\\|SymKod\\|"
        "Thue\\|Tup\\|UserProgram\\|VSrfK3\\|VSrfK3Seq\\|"
        "VarargNum\\|"
        "\\)\\>")
      'font-lock-variable-name-face)

      ;; Verbose modes
      (cons (concat "\\<\\("
        "Brandt\\|Code\\|ECPointCount\\|Factorization\\|Genus\\|"
        "JacobianGroup\\|ModularForms\\|Quaternion\\|Satoh\\|SEA\\|"
        "\\)\\>")
      'font-lock-reference-face)

      ;; Reserved words.
      (cons (concat "\\<\\("
        "assert\\|assigned\\|attributes\\|break\\|case\\|"
        "continue\\|declare\\|delete\\|do\\|elif\\|"
        "else\\|end\\|error\\|exit\\|for\\|"
        "forward\\|fprintf\\|function\\|getline\\|if\\|"
        "intrinsic\\|next\\|print\\|printf\\|procedure\\|"
        "repeat\\|require\\|requirege\\|requirerange\\|return\\|"
        "select\\|then\\|time\\|until\\|verbose\\|"
        "vprint\\|vprintf\\|where\\|when\\|while\\|"
        "\\)\\>")
      'font-lock-keyword-face)

      ;; Operators.
      (cons (mapconcat 'identity '(
        " not " " and " " or " " le " " lt " " ge " " gt " " in " 
        " notin " " cmpeq " " eq " " ne " " div " " mod " " join " " meet " 
        " cat " " true" " false" "#" "@" "&" "!" "<" 
        "|" ":->" "->" ">" )
        "\\|")
      'font-lock-type-face)

      ;; Punctuation.
      (cons (mapconcat 'identity '(
        " : " "::" "\;" )
        "\\|")
      'font-lock-constant-face)
    )
  )
"Default expressions to highlight in MAGMA mode.")

